Ext.define("flexunit.framework.Assert", function(Assert) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /
package flexunit.framework
{
   import mx.utils.ObjectUtil;
   import mx.utils.StringUtil;
   
   /**
    * Base class containing static assert methods.
    * /
   public class Assert
   {
      protected static var _assertionsMade : Number = 0;
      private static*/
    var _maxAssertionsMade$static = /* : Number*/
        0;
    /*
      private static*/
    var _totalAssertionsMade$static = /* : Number*/
        0;
    /*

      public static*/
    function resetAssertionsMade$static() /* : void*/
    {
        if (Assert._assertionsMade > _maxAssertionsMade$static) {
            _maxAssertionsMade$static = Assert._assertionsMade;
        }
        Assert._assertionsMade = 0;
    }
    /*
      
      public static*/
    function oneAssertionHasBeenMade$static() /* : void*/
    {
        Assert._assertionsMade++;
        _totalAssertionsMade$static++;
    }
    /*

      public static*/
    function resetEveryAsserionsFields$static() /* : void*/
    {
        Assert._assertionsMade = 0;
        _maxAssertionsMade$static = 0;
        _totalAssertionsMade$static = 0;
    }
    /*
            
      public static*/
    function get$maxAssertionsMade$static() /* : Number*/
    {
        return _maxAssertionsMade$static;
    }
    /*
      
      public static*/
    function get$assetionsMade$static() /* : Number*/
    {
        return Assert._assertionsMade;
    }
    /*
      
      public static*/
    function get$totalAssertionsMade$static() /* : Number*/
    {
        return _totalAssertionsMade$static;
    }
    /*
      
      public*/
    function Assert$() {}
    /*

   //---------------------------------------------------------------------------


      /**
       * Asserts that 2 values are equal.
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", expected, actual ]
       *
       * /
      public static*/
    function assertEquals$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 3) {
            failNotEquals$static(rest[0], rest[1], rest[2]);
        } else {
            failNotEquals$static("", rest[0], rest[1]);
        }
    }
    /*
      
   //---------------------------------------------------------------------------


      /**
       * Asserts that 2 objects are equal.
       * This method will recursively compare properties on the nested objects.
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", expected, actual ]
       *
       * /
      public static*/
    function assertObjectEquals$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 3) {
            failObjectEquals$static(rest[0], rest[1], rest[2]);
        } else {
            failObjectEquals$static("", rest[0], rest[1]);
        }
    }
    /* 


   //---------------------------------------------------------------------------

      /**
       * Asserts that a string matches a regexp.
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", regexp, stringToTest ]
       *
       * /
      public static*/
    function assertMatch$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 3) {
            failNoMatch$static(rest[0], rest[1], rest[2]);
        } else {
            failNoMatch$static("", rest[0], rest[1]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that a string doesn't match a regexp.
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", regexp, stringToTest ]
       *
       * /
      public static*/
    function assertNoMatch$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 3) {
            failMatch$static(rest[0], rest[1], rest[2]);
        } else {
            failMatch$static("", rest[0], rest[1]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that the first string is contained in the second one
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", shortString, longString ]
       *
       * /
      public static*/
    function assertContained$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 3) {
            failNotContained$static(rest[0], rest[1], rest[2]);
        } else {
            failNotContained$static("", rest[0], rest[1]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that the first string is not contained in the second one
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", shortString, longString ]
       *
       * /
      public static*/
    function assertNotContained$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 3) {
            failContained$static(rest[0], rest[1], rest[2]);
        } else {
            failContained$static("", rest[0], rest[1]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that two objects are strickly identical
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", expectedObject, actualObject ]
       *
       * /
      public static*/
    function assertStrictlyEquals$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 3) {
            failNotStrictlyEquals$static(rest[0], rest[1], rest[2]);
        } else {
            failNotStrictlyEquals$static("", rest[0], rest[1]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that a condition is true
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", condition ]
       *
       * /
      public static*/
    function assertTrue$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 2) {
            failNotTrue$static(rest[0], rest[1]);
        } else {
            failNotTrue$static("", rest[0]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that a condition is false
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", condition ]
       *
       * /
      public static*/
    function assertFalse$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 2) {
            failTrue$static(rest[0], rest[1]);
        } else {
            failTrue$static("", rest[0]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that an object is null
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", objectToTest ]
       *
       * /
      public static*/
    function assertNull$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 2) {
            failNotNull$static(rest[0], rest[1]);
        } else {
            failNotNull$static("", rest[0]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that an object is not null
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", objectToTest ]
       *
       * /
      public static*/
    function assertNotNull$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 2) {
            failNull$static(rest[0], rest[1]);
        } else {
            failNull$static("", rest[0]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that an object is undefined
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", objectToTest ]
       *
       * /
      public static*/
    function assertUndefined$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 2) {
            failNotUndefined$static(rest[0], rest[1]);
        } else {
            failNotUndefined$static("", rest[0]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Asserts that an object is not undefined
       * The first argument can be the message when the assertion fails
       *
       * @param rest [ message = "", objectToTest ]
       *
       * /
      public static*/
    function assertNotUndefined$static() /* ... rest*/
    /* : void*/
    {
        var rest = Array.prototype.slice.call(arguments);
        if (rest.length == 2) {
            failUndefined$static(rest[0], rest[1]);
        } else {
            failUndefined$static("", rest[0]);
        }
    }
    /*

   //---------------------------------------------------------------------------

      /**
       * Fails with the given message
       *
       * @param failMessage
       * /
      public static*/
    function fail$static(failMessage) /* : String = ""*/
    /* : void*/
    {
        if (arguments.length <= 0)  {
            failMessage = "";
        }
        
        throw new flexunit.framework.AssertionFailedError(failMessage);
    }
    /*

   //---------------------------------------------------------------------------

      private static*/
    function failNotEquals$static(message, /* : String*/
    expected, /* : Object*/
    actual) /* : Object*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (expected != actual) {
            if (AS3.is(expected, Number) && AS3.is(actual, Number) && isNaN(Number(expected)) && isNaN(Number(actual))) {
                return;
            }
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.EXPECTED_BUT_WAS, expected, actual));
        }
    }
    /*
      
      private static*/
    function failObjectEquals$static(message, /* : String*/
    expected, /* : Object*/
    actual) /* : Object*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (mx.utils.ObjectUtil.compare(expected, actual) != 0) {
            failWithUserMessage$static(message, flexunit.framework.AssertStringFormats.ACTUAL_OBJECT_DIFFERS);
        }
    }
    /*

      private static*/
    function failNoMatch$static(message, /* : String*/
    regexp, /* : RegExp*/
    string) /* : String*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (!regexp.test(string)) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.NO_MATCH, string, regexp.toString()));
        }
    }
    /*

      private static*/
    function failMatch$static(message, /* : String*/
    regexp, /* : RegExp*/
    string) /* : String*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (regexp.test(string)) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.MATCH, string, regexp.toString()));
        }
    }
    /*

      private static*/
    function failNotContained$static(message, /* : String*/
    shortString, /* : String*/
    longString) /* : String*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (longString.indexOf(shortString) < 0) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.NOT_CONTAINED, shortString, longString));
        }
    }
    /*

      private static*/
    function failContained$static(message, /* : String*/
    shortString, /* : String*/
    longString) /* : String*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (longString.indexOf(shortString) >= 0) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.CONTAINED, shortString, longString));
        }
    }
    /*

      private static*/
    function failNotStrictlyEquals$static(message, /* : String*/
    expected, /* : Object*/
    actual) /* : Object*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (expected !== actual) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.EXPECTED_BUT_WAS, expected, actual));
        }
    }
    /*

      private static*/
    function failNotTrue$static(message, /* : String*/
    condition) /* : Boolean*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (!condition) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.EXPECTED_BUT_WAS, true, false));
        }
    }
    /*

      private static*/
    function failTrue$static(message, /* : String*/
    condition) /* : Boolean*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (condition) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.EXPECTED_BUT_WAS, false, true));
        }
    }
    /*

      private static*/
    function failNull$static(message, /* : String*/
    object) /* : Object*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (object == null) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.NULL, object));
        }
    }
    /*

      private static*/
    function failNotNull$static(message, /* : String*/
    object) /* : Object*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (object != null) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.NOT_NULL, object));
        }
    }
    /*

      private static*/
    function failUndefined$static(message, /* : String*/
    object) /* : Object*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (object == null) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.UNDEFINED, object));
        }
    }
    /*

      private static*/
    function failNotUndefined$static(message, /* : String*/
    object) /* : Object*/
    /* : void*/
    {
        Assert.oneAssertionHasBeenMade();
        if (object != null) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.NOT_UNDEFINED, object));
        }
    }
    /*

      private static*/
    function failWithUserMessage$static(userMessage, /* : String*/
    failMessage) /* : String*/
    /* : void*/
    {
        if (userMessage.length > 0) {
            userMessage = userMessage + " - ";
        }
        throw new flexunit.framework.AssertionFailedError(userMessage + failMessage);
    }
    /*
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: Assert$,
        statics: {
            _assertionsMade: 0,
            resetAssertionsMade: resetAssertionsMade$static,
            oneAssertionHasBeenMade: oneAssertionHasBeenMade$static,
            resetEveryAsserionsFields: resetEveryAsserionsFields$static,
            assertEquals: assertEquals$static,
            assertObjectEquals: assertObjectEquals$static,
            assertMatch: assertMatch$static,
            assertNoMatch: assertNoMatch$static,
            assertContained: assertContained$static,
            assertNotContained: assertNotContained$static,
            assertStrictlyEquals: assertStrictlyEquals$static,
            assertTrue: assertTrue$static,
            assertFalse: assertFalse$static,
            assertNull: assertNull$static,
            assertNotNull: assertNotNull$static,
            assertUndefined: assertUndefined$static,
            assertNotUndefined: assertNotUndefined$static,
            fail: fail$static
        },
        __accessors__: {
            statics: {
                maxAssertionsMade: {
                    get: get$maxAssertionsMade$static
                },
                assetionsMade: {
                    get: get$assetionsMade$static
                },
                totalAssertionsMade: {
                    get: get$totalAssertionsMade$static
                }
            }
        },
        requires: [
            "mx.collections.IList",
            "mx.utils.ObjectUtil",
            "mx.utils.StringUtil"
        ],
        uses: [
            "flexunit.framework.AssertStringFormats",
            "flexunit.framework.AssertionFailedError"
        ]
    };
});

Ext.define("flexunit.framework.AssertStringFormats", function(AssertStringFormats) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /
package flexunit.framework
{
   public class AssertStringFormats
   {
      public static const ASYNC_CALL_NOT_FIRED : String = "Asynchronous function did not fire after {0} ms";
      public static const CONTAINED : String = "<{0}> found in <{1}>";
      public static const EVENT_DID_NOT_OCCUR : String = "Expected events <{0}> but caught events <{1}>";
      public static const EVENT_DID_OCCUR : String = "Did not expect events <{0}> but caught events <{1}>";
      public static const EXPECTED_BUT_WAS : String = "expected:<{0}> but was:<{1}>";
      public static const MATCH : String = "<{0}> matched <{1}>";
      public static const NO_MATCH : String = "<{0}> did not match <{1}>";
      public static const NOT_CONTAINED : String = "did not find <{0}> in <{1}>";
      public static const NOT_NULL : String = "object was not null: {0}";
      public static const NOT_UNDEFINED : String = "object was not undefined: {0}";
      public static const NULL : String = "object was null: {0}";
      public static const UNDEFINED : String = "object was undefined: {0}";
      public static const ACTUAL_OBJECT_DIFFERS : String = "At least one property of the actual object differs from the expected object";
   }*/
    function AssertStringFormats$() {}
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: AssertStringFormats$,
        statics: {
            ASYNC_CALL_NOT_FIRED: "Asynchronous function did not fire after {0} ms",
            CONTAINED: "<{0}> found in <{1}>",
            EVENT_DID_NOT_OCCUR: "Expected events <{0}> but caught events <{1}>",
            EVENT_DID_OCCUR: "Did not expect events <{0}> but caught events <{1}>",
            EXPECTED_BUT_WAS: "expected:<{0}> but was:<{1}>",
            MATCH: "<{0}> matched <{1}>",
            NO_MATCH: "<{0}> did not match <{1}>",
            NOT_CONTAINED: "did not find <{0}> in <{1}>",
            NOT_NULL: "object was not null: {0}",
            NOT_UNDEFINED: "object was not undefined: {0}",
            NULL: "object was null: {0}",
            UNDEFINED: "object was undefined: {0}",
            ACTUAL_OBJECT_DIFFERS: "At least one property of the actual object differs from the expected object"
        }
    };
});

Ext.define("flexunit.framework.AssertionFailedError", function(AssertionFailedError) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /
package flexunit.framework
{
   /**
    * Error thrown if an assertion fails.
    * /
   public class AssertionFailedError extends Error
   {
       public*/
    function AssertionFailedError$(message) /* : String = ""*/
    {
        if (arguments.length <= 0)  {
            message = "";
        }
        
        AS3.Error.prototype.constructor.call(this, message);
    }
    /*

   }
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "AS3.Error",
        constructor: AssertionFailedError$,
        requires: [
            "AS3.Error"
        ]
    };
});

Ext.define("flexunit.framework.AsyncTestHelper", function(AsyncTestHelper) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{
   import flash.events.*;
   import flash.utils.*;
   
   import mx.utils.StringUtil;

   public class AsyncTestHelper
   {
       public*/
    function AsyncTestHelper$(testCase, /* : TestCase*/
    testResult) /* : TestResult*/
    {
        this.testCase$1 = testCase;
        this.testResult$1 = testResult;
        this.timer$1 = new flash.utils.Timer(100);
        this.timer$1.addEventListener(flash.events.TimerEvent.TIMER, AS3.bind(this, "timerHandler"));
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function startAsync() /* : void*/
    {
        this.loadAsync();
        if (this.objToPass$1 != null) {
            this.testResult$1.continueRun(this.testCase$1);
        } else {
            this.timer$1.start();
        }
    }
    /*

   //------------------------------------------------------------------------------


       public*/
    function loadAsync() /* : void*/
    {
        var async = /* : Object*/
            this.testCase$1.getNextAsync();
        this.func$1 = async.func;
        this.extraData$1 = async.extraData;
        this.failFunc$1 = async.failFunc;
        //BUG 114824 WORKAROUND
        this.timer$1 = new flash.utils.Timer(async.timeout, 1);
        this.timer$1.addEventListener(flash.events.TimerEvent.TIMER, AS3.bind(this, "timerHandler"));
        //END WORKAROUND
        this.timer$1.delay = async.timeout;
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function runNext() /* : void*/
    {
        if (this.shouldFail$1) {
            if (this.failFunc$1 != null) {
                this.failFunc$1(this.extraData$1);
            } else {
                flexunit.framework.Assert.fail(mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.ASYNC_CALL_NOT_FIRED, this.timer$1.delay));
            }
        } else {
            if (this.extraData$1 != null) {
                this.func$1(this.objToPass$1, this.extraData$1);
            } else {
                this.func$1(this.objToPass$1);
            }
            this.func$1 = null;
            this.objToPass$1 = null;
            this.extraData$1 = null;
        }
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function timerHandler(event) /* : TimerEvent*/
    /* : void*/
    {
        this.timer$1.stop();
        this.shouldFail$1 = true;
        this.testResult$1.continueRun(this.testCase$1);
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function handleEvent(event) /* : Object*/
    /* : void*/
    {
        var wasReallyAsync = /* : Boolean*/
            this.timer$1.running;
        this.timer$1.stop();
        if (this.shouldFail$1)  {
            return;
        }
        
        this.objToPass$1 = event;
        if (wasReallyAsync) {
            this.testResult$1.continueRun(this.testCase$1);
        }
    }
    /*

   //------------------------------------------------------------------------------

       //IResponder methods here (they'd look similar to handleEvent) ...

   //------------------------------------------------------------------------------

       private var testCase : TestCase;
       private var func : Function;
       private var extraData : Object;
       private var failFunc : Function;
       private var testResult : TestResult;

       private var shouldFail : Boolean = false;
       private var objToPass : Object;

       private var timer : Timer;

   }
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: AsyncTestHelper$,
        startAsync: startAsync,
        loadAsync: loadAsync,
        runNext: runNext,
        timerHandler: timerHandler,
        handleEvent: handleEvent,
        testCase$1: null,
        func$1: null,
        extraData$1: null,
        failFunc$1: null,
        testResult$1: null,
        shouldFail$1: false,
        objToPass$1: null,
        timer$1: null,
        requires: [
            "flash.events.Event",
            "flash.events.EventDispatcher",
            "flash.events.IEventDispatcher",
            "flash.events.TimerEvent",
            "flash.utils.Timer",
            "flash.utils.clearInterval",
            "flash.utils.setInterval",
            "mx.utils.StringUtil"
        ],
        uses: [
            "flexunit.framework.Assert",
            "flexunit.framework.AssertStringFormats"
        ]
    };
});

Ext.define("flexunit.framework.Reflective", function(Reflective) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{
   /**
    * Interface for basic reflection implementation.
    * /
   public interface Reflective
   {
       //AS3 should allow these to be a getter/setter pair but I'm having trouble right now.
      function get className() : String;
   }
}

============================================== Jangaroo part ==============================================*/
    return {};
});

Ext.define("flexunit.framework.Test", function(Test) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{

   /**
    * A <code>Test</code> can be run, with the results collected.
    *
    * @see TestResult
    * /
   public interface Test extends Reflective
   {
     /**
      * Runs the tests, populating the <code>result</code> parameter.
      * @param result The TestResult instance to be populated
      * /
      function runWithResult( result : TestResult ) : void;


      /**
      * The number of test cases in this test.
      * @return A Number representing the count of test cases in this test.
      * /
       function countTestCases() : Number;

      /**
      * Returns the names of the test methods for this test.
      * @return  Array of method names for this test
      * /
      function getTestMethodNames() : Array;

      function toString() : String;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.framework.Reflective"
        ],
        requires: [
            "flexunit.framework.Reflective"
        ]
    };
});

Ext.define("flexunit.framework.TestCase", function(TestCase) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /
package flexunit.framework
{

   import flash.utils.*;

   /**
    * The Base Class for test cases. A Test case defines the fixture in which to run multiple tests.
    * @description
    * Each test runs in its own fixture so there can be no side effects among test runs.
    * Here is an example:
    * <code>
    * import samples.com.iterationtwo.Money;
    * import flexunit.framework.TestCase;
    *
    * class samples.test.com.iterationtwo.TestMoney extends TestCase
    * {
    *     ...
    * }
    * </code>
    *
    * For each test, implement a method which interacts
    * with the fixture. Verify the expected results using the assertions
    * residing in the base class <code>Assert</code>.
    * <code>
    *  public function testAddMoney()
    *  {
    *     var dollars1 : Number = 3;
    *     var cents1 : Number = 50;
    *     var money1 : Money = new Money( dollars1, cents1 );
    *
    *     var dollars2 : Number = 3;
    *     var cents2 : Number = 20;
    *     var money2 : Money = new Money( dollars2, cents2 );
    *
    *     var money3 : Money = money1.addMoney( money2 )
    *
    *     assertNotNull( "money was null", money3 );
    *     assertNotUndefined( "money was undefined", money3 );
    *
    *     assertEquals( "Dollars should be 6", 6, money3.dollars );
    *     assertEquals( "Cents should be 70", 70, money3.cents );
    *  }
    * </code>
    * You can also specify <code>setUp()</code> and <code>tearDown()</code> methods
    * to initialize and clean-up tests.
    * <code>
    * import samples.com.iterationtwo.Money;
    * import flexunit.framework.TestCase;
    *
    * class samples.test.com.iterationtwo.TestMoneyWithSetUpTearDown extends TestCase
    * {
    *    public function setUp() : void
    *    {
    *       money = new Money( 3, 50 );
    *    }
    *
    *    public function tearDown() : void
    *    {
    *       money = null;
    *    }
    *
    *    private var money : Money;
    * }
    * </code>
    * The tests to be run can be collected into a <code>TestSuite</code> and run using a a <code>TestRunner</code>.
    *
    * The AS3 version has been extended to allow asynchronous tests.  This means that an individual test method
    * will not be considered complete until any subsequent asynchronous calls that have been specified have completed.
    * Adding an asynchronous callback is easy, simply call the addAsync method passing in the callback method and the
    * time in which it is expected to complete.  You can also pass any data you want carried through the call.  addAsync
    * will return a function capable of handling generic events which you can then set as the event listener on an object.
    * E.g.,
    * <code>
    *   public function testAsync() : void
    *   {
    *      var myDispatcher : ValueDispatcher = new ValueDispatcher();
    *      var helper : Function = addAsync(eventHandler, 200, "foo");
    *      myDispatcher.addEventListener("value", helper);
    *      myDispatcher.dispatchValue("foo", 100); //the dispatcher should send a value in 100 ms
    *   }
    *
    *   //the expected is the extra data that was passed in addAsync above
    *   public function eventHandler(event : ValueEvent, expected : String)
    *   {
    *      var actual : String = event.value;
    *      assertEquals(expected, actual);
    *   }
    * <code>
    *
    *
    * @see flexunit.framework.Assert
    * @see flexunit.framework.TestSuite
    * @see flexunit.framework.TestResult
    *
    * /
   public class TestCase extends Assert implements Test
   {

   //------------------------------------------------------------------------------

      /**
      * The TestCase constructor. If you provide a contstructor in a <code>TestCase</code> subclass,
      * you should ensure that this constructor is called.
      * @param methodName The name of the test method to be called in the test run.
      * /
      public*/
    function TestCase$(methodName) /* : String = null*/
    {
        if (arguments.length <= 0)  {
            methodName = null;
        }
        
        flexunit.framework.Assert.prototype.constructor.call(this);
        this.methodName = methodName;
        this.asyncMethods$2 = new Array();
    }
    /*

   //------------------------------------------------------------------------------

      /**
      * Creates a new <code>TestResult</code> and runs the tests, populating that <code>TestResult</code>
      * with the results.
      * /
       public*/
    function run() /* : TestResult*/
    {
        var result = /* : TestResult*/
            new flexunit.framework.TestResult();
        this.runWithResult(result);
        return result;
    }
    /*

   //------------------------------------------------------------------------------

      /**
      * Runs the tests, populating the <code>result</code> parameter.
      * @param result The TestResult instance to be populated
      * /
       public*/
    function runWithResult(result) /* : TestResult*/
    /* : void*/
    {
        result.run(this);
    }
    /*


   //------------------------------------------------------------------------------

      /**
      * Runs <code>setUp()</code>
      * /
       public*/
    function runStart() /* : void*/
    {
        this.setUp();
    }
    /*

   //------------------------------------------------------------------------------

      /**
       * Runs the normal test method or the next asynchronous method
       * /
       public*/
    function runMiddle() /* : void*/
    {
        this.runTestOrAsync$2();
    }
    /*

   //------------------------------------------------------------------------------

      /**
       * Runs <code>tearDown()</code>
       * /
       public*/
    function runFinish() /* : void*/
    {
        this.asyncTestHelper$2 = null;
        this.tearDown();
    }
    /*

   //------------------------------------------------------------------------------

      /**
       * Empty implementation of <code>setUp()</code>. Can be overridden in test class.
       * /
       public*/
    function setUp() /* : void*/
    {}
    /*

   //------------------------------------------------------------------------------

      /**
       * Empty implementation of <code>tearDown()</code>. Can be overridden in test class.
       * /
       public*/
    function tearDown() /* : void*/
    {}
    /*

   //------------------------------------------------------------------------------

      /**
       * The number of test cases in this test class.
       * @return A Number representation the count of test cases in this test class. Always returns 1 for <code>TestCase</code>
       * /
       public*/
    function countTestCases() /* : Number*/
    {
        return 1;
    }
    /*

   //------------------------------------------------------------------------------

      /**
       * A string representation of the test case
       * @return A string representation of the test class name and the test method name.
       * /
       public*/
    function toString() /* : String*/
    {
        return this.methodName + " (" + this.className + ")";
    }
    /*

   //------------------------------------------------------------------------------

      /**
       * Returns the the fully qualified class name
       * @return The fully qualified class name
       * /
       public*/
    function get$className() /* : String*/
    {
        return flash.utils.getQualifiedClassName(this).replace(/::/, ".");
    }
    /*
       
       public*/
    function get$assertionsMade() /* : Number*/
    {
        return this._assertionsMade$2;
    }
    /*

   //------------------------------------------------------------------------------

      /**
       * Runs the test or if there is an AsynchronousTestHelper runs the next asynchronous method
       * /
       private*/
    function runTestOrAsync() /* : void*/
    {
        if (this.methodName == null || this.methodName == "") {
            flexunit.framework.Assert.fail("No test method to run");
        }
        if (this.asyncTestHelper$2 != null) {
            this.asyncTestHelper$2.runNext();
        } else {
            this[this.methodName]();
        }
    }
    /*

   //------------------------------------------------------------------------------

      /**
      * Add an asynchronous check point to the test.
      * This method will return an event handler function.
      *
      * @param func the Function to execute when things have been handled
      * @param timeout if the function isn't called within this time the test is considered a failure
      * @param passThroughData data that will be passed to your function (only if non-null) as the 2nd argument
      * @param failFunc a Function that will be called if the asynchronous function fails to execute, useful if perhaps the failure to
      *     execute was intentional or if you want a specific failure message
      * @return the Function that can be used as an event listener
      * /
      public*/
    function addAsync(func, /* : Function*/
    timeout, /* : int*/
    passThroughData, /* : Object = null*/
    failFunc) /* : Function = null*/
    /* : Function*/
    {
        switch (Math.max(arguments.length, 2)) {
            case 2:
                passThroughData = null;
            case 3:
                failFunc = null;
        }
        flexunit.framework.Assert.oneAssertionHasBeenMade();
        if (this.asyncTestHelper$2 == null) {
            this.asyncTestHelper$2 = new flexunit.framework.AsyncTestHelper(this, this.testResult$2);
        }
        this.asyncMethods$2.push({
            func: func,
            timeout: timeout,
            extraData: passThroughData,
            failFunc: failFunc
        });
        return AS3.bind(this.asyncTestHelper$2, "handleEvent");
    }
    /*

   //------------------------------------------------------------------------------

      /**
       * Returns true if there are any asynchronous methods remaining to be called
       * /
       public*/
    function hasAsync() /* : Boolean*/
    {
        return this.asyncMethods$2.length > 0;
    }
    /*

   //------------------------------------------------------------------------------

      /**
       * Called by the TestResult to kick off wait for the next asynchronous method
       * /
        public*/
    function startAsync() /* : void*/
    {
        this.asyncTestHelper$2.startAsync();
    }
    /*

   //------------------------------------------------------------------------------
      /**
       * The AsyncTestHelper will call this when it's ready for to start the next async.  It's possible that
       * it will need to get access to it even before async has been started if the call didn't actually end
       * up being asynchronous.
       * /
       public*/
    function getNextAsync() /* : Object*/
    {
        return this.asyncMethods$2.shift();
    }
    /*

   //------------------------------------------------------------------------------

      /**
       * Called by the TestResult to pass along so that it can be passed for async
       * /
       public*/
    function setTestResult(result) /* : TestResult*/
    /* : void*/
    {
        this.testResult$2 = result;
    }
    /*
       
       public*/
    function set$assertionsMade(value) /* : Number*/
    /* : void*/
    {
        this._assertionsMade$2 = value;
    }
    /*
   //------------------------------------------------------------------------------

      public*/
    function getTestMethodNames() /* : Array*/
    {
        if (this.methodNames$2 == null) {
            this.methodNames$2 = new Array();
            var type = /* : XML*/
                flash.utils.describeType(this);
            var names = /* : XMLList*/
                type.method["@name"];
            for (var i = /* : uint*/
                0; i < names.length(); i++) {
                if (this.isTestMethod$2(String(names[i]))) {
                    this.methodNames$2.push(String(names[i]));
                }
            }
        }
        return this.methodNames$2;
    }
    /*

      private*/
    function isTestMethod(name) /* : String*/
    /* : Boolean*/
    {
        return name.indexOf("test", 0) == 0;
    }
    /*

   //------------------------------------------------------------------------------

      /**
       *   An array of all the test methods for this test case.
       * /
       private var methodNames : Array;

      /**
       * The method name of the individual test to be run
       * /
       public var methodName : String;
       private var asyncMethods : Array;
       private var asyncTestHelper : AsyncTestHelper;
       private var testResult : TestResult;
       private var _assertionsMade : Number = 0;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "flexunit.framework.Assert",
        mixins: [
            "flexunit.framework.Test"
        ],
        constructor: TestCase$,
        run: run,
        runWithResult: runWithResult,
        runStart: runStart,
        runMiddle: runMiddle,
        runFinish: runFinish,
        setUp: setUp,
        tearDown: tearDown,
        countTestCases: countTestCases,
        toString: toString,
        runTestOrAsync$2: runTestOrAsync,
        addAsync: addAsync,
        hasAsync: hasAsync,
        startAsync: startAsync,
        getNextAsync: getNextAsync,
        setTestResult: setTestResult,
        getTestMethodNames: getTestMethodNames,
        isTestMethod$2: isTestMethod,
        methodNames$2: null,
        methodName: null,
        asyncMethods$2: null,
        asyncTestHelper$2: null,
        testResult$2: null,
        _assertionsMade$2: 0,
        __accessors__: {
            className: {
                get: get$className
            },
            assertionsMade: {
                get: get$assertionsMade,
                set: set$assertionsMade
            }
        },
        requires: [
            "flash.utils.describeType",
            "flash.utils.getQualifiedClassName",
            "flexunit.framework.Assert",
            "flexunit.framework.Reflective",
            "flexunit.framework.Test"
        ],
        uses: [
            "flexunit.framework.AsyncTestHelper",
            "flexunit.framework.TestResult"
        ]
    };
});

Ext.define("flexunit.framework.EventfulTestCase", function(EventfulTestCase) {
    /* /*

Copyright (c) 2008. Adobe Systems Incorporated.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Adobe Systems Incorporated nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

@ignore
* /

package flexunit.framework
{
   import flash.events.Event;
   import flash.events.IEventDispatcher;
   
   import mx.utils.StringUtil;

   /**
    * A base class for test cases that are interested in event dispatching.
    *
    * <p>
    * The process of testing the dispatch of events typically involves four
    * stages:
    * </p>
    *
    * <ol>
    *    <li>
    *       Record the events that are expected to occur or not expected to occur
    *       during the test case using the <code>listenForEvent()</code> function.
    *    </li>
    *    <li>
    *       Perform the action under test in the same way as a normal test case.
    *    </li>
    *    <li>
    *       Assert that the expected events actually occurred using the
    *       <code>assertEvents()</code> function.
    *    </li>
    *    <li>
    *       Assert that the details of these events are correct using the
    *       <code>lastDispatchedEvent</code> and <code>dispatchedEvents</code> property
    *       getters and standard FlexUnit assertion functions.
    *    </li>
    * </ol>
    *
    * <p>
    * An example test case is shown below:
    * </p>
    *
    * <pre>
    * public function testAddPlutonium() : void
    * {
    *    // record the expected and unexpected events
    *    listenForEvent( model, TemperatureChangeEvent.TEMPERATURE_CHANGED );
    *    listenForEvent( model, EmergencyEvent.NUCLEAR_MELTDOWN, UNEXPECTED );
    *
    *    // perform the action that is being tested
    *    reactor.addPlutonium( plutonium );
    *
    *    // assert that the expected events were dispatched and the unexpected
    *    // events were not dispatched
    *    assertEvents(
    *       "The temperature change event was not dispatched or the world has ended" );
    *
    *    // assert on the details of the actual event
    *    var temperatureChangeEvent : TemperatureChangeEvent =
    *       TemperatureChangeEvent( lastDispatchedEvent );
    *
    *    var expectedTemperature : Number = 550;
    *
    *    assertEquals(
    *       "The expected temperature change did not occur.",
    *       expectedTemperature,
    *       temperatureChangeEvent.temperature );
    * }
    * </pre>
    * /
   public class EventfulTestCase extends TestCase
   {
   	/** Boolean to indicate the event is expected to be dispatched * /
   	public static const EVENT_EXPECTED : Boolean = true;

   	/** Boolean to indicate the event is not expected to be dispatched * /
   	public static const EVENT_UNEXPECTED : Boolean = false;
   	
      /** Listens for expected events during a single test case. * /
      private var eventListener : EventfulTestCaseListener =*/
    function eventListener_() {
        this.eventListener$3 = (new flexunit.framework.EventfulTestCaseListener());
    }
    /*;

      //-------------------------------
      //
      // actual event getters
      //
      //-------------------------------

      /**
       * Gets the last event to actually be heard. Only expected events
       * registered through the <code>listenForEvent()</code> function
       * can be heard.
       * /
      protected*/
    function get$lastDispatchedExpectedEvent() /* : Event*/
    {
        return this.eventListener$3.lastDispatchedExpectedEvent;
    }
    /*

      /**
       * Gets the events that were heard.
       * /
      protected*/
    function get$dispatchedExpectedEvents() /* : Array*/
    {
        return this.eventListener$3.dispatchedExpectedEvents;
    }
    /*

      //-------------------------------
      //
      // constructor
      //
      //-------------------------------

      public*/
    function EventfulTestCase$(methodName) /* : String = null*/
    {
        if (arguments.length <= 0)  {
            methodName = null;
        }
        
        flexunit.framework.TestCase.prototype.constructor.call(this, methodName);
        eventListener_.call(this);
        
    }
    /*

      //-------------------------------
      //
      // listen functions
      //
      //-------------------------------

      /**
       * Listens for an. When the <tt>assertEvents()</tt>
       * function is called, the events will be compared with the
       * actual events.
       *
       * @param source
       *    the object that is to be listened on for the dispatched event
       * @param type
       *    the type of event that the source object might dispatch
       * @param expected
       *    whether the event is expected to be dispatched or now; defaults to <code>EVENT_EXPECTED</code>
       * /
      protected*/
    function listenForEvent(source, /* : IEventDispatcher*/
    type, /* : String*/
    expected) /* : Boolean = EventfulTestCase.EVENT_EXPECTED*/
    /* : void*/
    {
        if (arguments.length <= 2)  {
            expected = EventfulTestCase.EVENT_EXPECTED;
        }
        
        // prevent the same event listener hearing an event multiple times
        if (source.hasEventListener(type)) {
            source.removeEventListener(type, AS3.bind(this.eventListener$3, "handleEvent"));
        }
        this.eventListener$3.listenForEvent(type, expected);
        source.addEventListener(type, AS3.bind(this.eventListener$3, "handleEvent"), false, 0, true);
    }
    // weak reference so singleton listeners are cleaned up
    /*

      //-------------------------------
      //
      // assertions
      //
      //-------------------------------

      /**
       * Asserts that the expected events were dispatched and the 
       * unexpected events were not dispatched. The events
       * must first be recorded using the <tt>listenForEvents()</tt>
       * method.
       *
       * @param message
       *    the user message to display when a test failure occurs
       * /
      public*/
    function assertEvents(message) /* : String = ""*/
    /* : void*/
    {
        if (arguments.length <= 0)  {
            message = "";
        }
        
        flexunit.framework.Assert.oneAssertionHasBeenMade();
        var failed = /* : Boolean*/
            this.failExpectedEventsDidNotOccur$3(message, this.eventListener$3);
        if (!failed)  {
            this.failUnexpectedEventsOccur$3(message, this.eventListener$3);
        }
        
    }
    /*

      //-------------------------------
      //
      // private fail functions
      //
      //-------------------------------

      /**
       * Fails the test case if the expected events did not actually occur.
       * /
      private*/
    function failExpectedEventsDidNotOccur(message, /* : String*/
    listener) /* : EventfulTestCaseListener*/
    /* : Boolean*/
    {
        if (listener.expectedEventsDispatched() == false) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.EVENT_DID_NOT_OCCUR, listener.expectedEventTypes, listener.dispatchedExpectedEventTypes));
            return true;
        }
        return false;
    }
    /*

      /**
       * Fails the test case if the expected events did not actually occur.
       * /
      private*/
    function failUnexpectedEventsOccur(message, /* : String*/
    listener) /* : EventfulTestCaseListener*/
    /* : Boolean*/
    {
        if (listener.unexpectedEventsNotDispatched() == false) {
            failWithUserMessage$static(message, mx.utils.StringUtil.substitute(flexunit.framework.AssertStringFormats.EVENT_DID_OCCUR, listener.unexpectedEventTypes, listener.dispatchedExpectedEventTypes));
            return true;
        }
        return false;
    }
    /*

      /**
       * Private function duplicated from <code>Assert</code>. Ideally this
       * would be protected in the FlexUnit <code>Assert</code> class instead.
       * /
      private static*/
    function failWithUserMessage$static(userMessage, /* : String*/
    failMessage) /* : String*/
    /* : void*/
    {
        if (userMessage.length > 0) {
            userMessage += " - ";
        }
        throw new flexunit.framework.AssertionFailedError(userMessage + failMessage);
    }
    /*
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "flexunit.framework.TestCase",
        constructor: EventfulTestCase$,
        listenForEvent: listenForEvent,
        assertEvents: assertEvents,
        failExpectedEventsDidNotOccur$3: failExpectedEventsDidNotOccur,
        failUnexpectedEventsOccur$3: failUnexpectedEventsOccur,
        statics: {
            EVENT_EXPECTED: true,
            EVENT_UNEXPECTED: false
        },
        __accessors__: {
            lastDispatchedExpectedEvent: {
                get: get$lastDispatchedExpectedEvent
            },
            dispatchedExpectedEvents: {
                get: get$dispatchedExpectedEvents
            }
        },
        requires: [
            "flexunit.framework.Assert",
            "flexunit.framework.Reflective",
            "flexunit.framework.Test",
            "flexunit.framework.TestCase",
            "mx.utils.StringUtil"
        ],
        uses: [
            "flexunit.framework.AssertStringFormats",
            "flexunit.framework.AssertionFailedError",
            "flexunit.framework.EventfulTestCaseListener"
        ]
    };
});

Ext.define("flexunit.framework.EventfulTestCaseListener", function(EventfulTestCaseListener) {
    /* /*

Copyright (c) 2008. Adobe Systems Incorporated.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of Adobe Systems Incorporated nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

@ignore
* /

package flexunit.framework
{
   import flash.events.Event;

   import mx.collections.ArrayCollection;
   import mx.collections.ListCollectionView;

   /**
    * Listens for expected events, keeping track of the expected events that
    * actually occur. A helper class designed specifically for the
    * <code>EventfulTestCase</code>.
    * /
   internal class EventfulTestCaseListener
   {
      //-------------------------------
      //
      // properties
      //
      //-------------------------------

      private var _expectedEventTypes : ListCollectionView
               =*/
    function _expectedEventTypes_() {
        this._expectedEventTypes$1 = (new mx.collections.ArrayCollection());
    }
    /*;
      private var _unexpectedEventTypes : ListCollectionView
               =*/
    function _unexpectedEventTypes_() {
        this._unexpectedEventTypes$1 = (new mx.collections.ArrayCollection());
    }
    /*;

      private var _dispatchedExpectedEvents : ListCollectionView
               =*/
    function _dispatchedExpectedEvents_() {
        this._dispatchedExpectedEvents$1 = (new mx.collections.ArrayCollection());
    }
    /*;

      /**
       * Gets a comma-separated string listing the types of events that were
       * expected.
       * /
      public*/
    function get$expectedEventTypes() /* : String*/
    {
        var eventTypes = /* : String*/
            "";
        for (var i = /*: uint*/
            0; i < this._expectedEventTypes$1.length; i++) {
            eventTypes += AS3.as(this._expectedEventTypes$1[i], String);
            if (i < this._expectedEventTypes$1.length - 1) {
                eventTypes += ',';
            }
        }
        return eventTypes;
    }
    /*

      /**
       * Gets a comma-separated string listing the types of events that were not
       * expected.
       * /
      public*/
    function get$unexpectedEventTypes() /* : String*/
    {
        var eventTypes = /* : String*/
            "";
        for (var i = /*: uint*/
            0; i < this._unexpectedEventTypes$1.length; i++) {
            eventTypes += AS3.as(this._unexpectedEventTypes$1[i], String);
            if (i < this._unexpectedEventTypes$1.length - 1) {
                eventTypes += ',';
            }
        }
        return eventTypes;
    }
    /*

      /**
       * Gets a comma-separated string listing the types of events that have
       * been heard.
       * /
      public*/
    function get$dispatchedExpectedEventTypes() /* : String*/
    {
        var eventTypes = /* : String*/
            "";
        for (var i; /*: uint*/
        i < this._dispatchedExpectedEvents$1.length; i++) {
            var event = /* : Event*/
                AS3.as(this._dispatchedExpectedEvents$1[i], flash.events.Event);
            eventTypes += event.type;
            if (i < this._dispatchedExpectedEvents$1.length - 1) {
                eventTypes += ',';
            }
        }
        return eventTypes;
    }
    /*

      /**
       * Gets an array of all the events that have been heard.
       * /
      public*/
    function get$dispatchedExpectedEvents() /* : Array*/
    {
        return this._dispatchedExpectedEvents$1.toArray();
    }
    /*

      /**
       * Gets the last event to have been heard.
       * /
      public*/
    function get$lastDispatchedExpectedEvent() /* : Event*/
    {
        if (this._dispatchedExpectedEvents$1.length == 0) {
            return null;
        }
        return AS3.cast(flash.events.Event, this._dispatchedExpectedEvents$1.getItemAt(this._dispatchedExpectedEvents$1.length - 1));
    }
    /*

      //-------------------------------
      //
      // constructor
      //
      //-------------------------------

      public*/
    function EventfulTestCaseListener$() {
        _expectedEventTypes_.call(this);
        _unexpectedEventTypes_.call(this);
        _dispatchedExpectedEvents_.call(this);
        this._dispatchedExpectedEvents$1 = new mx.collections.ArrayCollection();
        this._expectedEventTypes$1 = new mx.collections.ArrayCollection();
    }
    /*

      //-------------------------------
      //
      // functions
      //
      //-------------------------------

      /**
       * Records an event being listened for.
       *
       * @param type
       *    the type of event
       * @param expected
       *    whether the event is expected to be dispatched or not
       * /
      public*/
    function listenForEvent(type, /* : String*/
    expected) /* : Boolean*/
    /* : void*/
    {
        if (expected)  {
            this._expectedEventTypes$1.addItem(type);
        }
        else  {
            this._unexpectedEventTypes$1.addItem(type);
        }
        
    }
    /*

      /**
       * Verifies that the expected events were heard, returning
       * <code>true</code> if so or <code>false</code> otherwise.
       * /
      public*/
    function expectedEventsDispatched() /* : Boolean*/
    {
        for (var i = /*:uint*/
            0; i < this._expectedEventTypes$1.length; ++i) {
            var expectedEvent = /* : String*/
                this._expectedEventTypes$1[i];
            if (this.expectedEventDispatched(expectedEvent) == false)  {
                return false;
            }
            
        }
        return true;
    }
    /*

      /**
      *  Verifies the expected event was dispatched
      * /
      public*/
    function expectedEventDispatched(expectedEvent) /* : String*/
    /* : Boolean*/
    {
        for (var i = /*:uint*/
            0; i < this._dispatchedExpectedEvents$1.length; ++i) {
            var actualEvent = /* : Event*/
                this._dispatchedExpectedEvents$1[i];
            if (actualEvent.type == expectedEvent) {
                return true;
            }
        }
        return false;
    }
    /*

      /**
       * Verifies that the unexpected events were not heard, returning
       * <code>true</code> if so or <code>false</code> otherwise.
       * /
      public*/
    function unexpectedEventsNotDispatched() /* : Boolean*/
    {
        for (var i = /*:uint*/
            0; i < this._unexpectedEventTypes$1.length; ++i) {
            var unexpectedEvent = /* : String*/
                this._unexpectedEventTypes$1[i];
            if (this.unexpectedEventNotDispatched(unexpectedEvent) == false) {
                return false;
            }
        }
        return true;
    }
    /*

      /**
      *  Verifies that the unexpected event was not dispatched
      * /
      public*/
    function unexpectedEventNotDispatched(unexpectedEvent) /* : String*/
    /* : Boolean*/
    {
        for (var i = /*:uint*/
            0; i < this._dispatchedExpectedEvents$1.length; ++i) {
            var actualEvent = /* : Event*/
                this._dispatchedExpectedEvents$1[i];
            if (actualEvent.type == unexpectedEvent) {
                return false;
            }
        }
        return true;
    }
    /*

      /**
       * Handles an event by recording that it actually occurred.
       * /
      public*/
    function handleEvent(event) /* : Event*/
    /* : void*/
    {
        this._dispatchedExpectedEvents$1.addItem(event);
    }
    /*
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: EventfulTestCaseListener$,
        listenForEvent: listenForEvent,
        expectedEventsDispatched: expectedEventsDispatched,
        expectedEventDispatched: expectedEventDispatched,
        unexpectedEventsNotDispatched: unexpectedEventsNotDispatched,
        unexpectedEventNotDispatched: unexpectedEventNotDispatched,
        handleEvent: handleEvent,
        __accessors__: {
            expectedEventTypes: {
                get: get$expectedEventTypes
            },
            unexpectedEventTypes: {
                get: get$unexpectedEventTypes
            },
            dispatchedExpectedEventTypes: {
                get: get$dispatchedExpectedEventTypes
            },
            dispatchedExpectedEvents: {
                get: get$dispatchedExpectedEvents
            },
            lastDispatchedExpectedEvent: {
                get: get$lastDispatchedExpectedEvent
            }
        },
        requires: [
            "flash.events.Event",
            "mx.collections.ArrayCollection",
            "mx.collections.IList",
            "mx.collections.ListCollectionView"
        ]
    };
});

Ext.define("flexunit.framework.Protectable", function(Protectable) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{
   /**
    * A Protectable instance can run and throw an Error.
    * /
   public interface Protectable
   {
      function protect() : void;
   }
}

============================================== Jangaroo part ==============================================*/
    return {};
});

Ext.define("flexunit.framework.ProtectedFinishTestCase", function(ProtectedFinishTestCase) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{
   /**
    * Used to finish the TestCase (this should generally be the tearDown method)
    * @see flexunit.framework.TestCase
    * /
   public class ProtectedFinishTestCase implements Protectable
   {
       public*/
    function ProtectedFinishTestCase$(testCase) /* : TestCase*/
    {
        this.testCase$1 = testCase;
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function protect() /* : void*/
    {
        this.testCase$1.runFinish();
    }
    /*

   //------------------------------------------------------------------------------

       private var testCase : TestCase;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.framework.Protectable"
        ],
        constructor: ProtectedFinishTestCase$,
        protect: protect,
        testCase$1: null,
        requires: [
            "flexunit.framework.Protectable"
        ]
    };
});

Ext.define("flexunit.framework.ProtectedMiddleTestCase", function(ProtectedMiddleTestCase) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{
   /**
    * Run the main part of the test, either the actual test or subsequent asynchronous callbacks.
    * @see flexunit.framework.TestCase
    * /
   public class ProtectedMiddleTestCase implements Protectable
   {
       public*/
    function ProtectedMiddleTestCase$(testCase) /* : TestCase*/
    {
        this.testCase$1 = testCase;
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function protect() /* : void*/
    {
        this.testCase$1.runMiddle();
    }
    /*

   //------------------------------------------------------------------------------

       private var testCase : TestCase;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.framework.Protectable"
        ],
        constructor: ProtectedMiddleTestCase$,
        protect: protect,
        testCase$1: null,
        requires: [
            "flexunit.framework.Protectable"
        ]
    };
});

Ext.define("flexunit.framework.ProtectedStartTestCase", function(ProtectedStartTestCase) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{
   /**
    * Run the beginning of a test, usually the setUp method
    * @see flexunit.framework.TestCase
    * /
   public class ProtectedStartTestCase implements Protectable
   {
       public*/
    function ProtectedStartTestCase$(testCase) /* : TestCase*/
    {
        this.testCase$1 = testCase;
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function protect() /* : void*/
    {
        this.testCase$1.runStart();
    }
    /*

   //------------------------------------------------------------------------------

       private var testCase : TestCase;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.framework.Protectable"
        ],
        constructor: ProtectedStartTestCase$,
        protect: protect,
        testCase$1: null,
        requires: [
            "flexunit.framework.Protectable"
        ]
    };
});

Ext.define("flexunit.framework.TestFailure", function(TestFailure) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{

   /**
    * A <code>TestFailure</code> collects a failed test together with
    * the caught Error.
    *
    * see TestResult
    * /
   public class TestFailure
   {
       public*/
    function TestFailure$(test, /* : Test*/
    error) /* : Error*/
    {
        this.test$1 = test;
        this.error$1 = error;
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function failedTest() /* : Test*/
    {
        return this.test$1;
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function thrownException() /* : Error*/
    {
        return this.error$1;
    }
    /*

   //------------------------------------------------------------------------------
       public*/
    function exceptionMessage() /* : String*/
    {
        if (!this.isFailure()) {
            return this.error$1.toString();
        } else {
            return this.error$1.toString();
        }
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function isFailure() /* : Boolean*/
    {
        return AS3.is(this.thrownException(), flexunit.framework.AssertionFailedError);
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function toString() /* : String*/
    {
        var errorMessage = /* : String*/
            Object(this.test$1).toString() + ": " + this.error$1.toString();
        if (!this.isFailure())  {
            errorMessage += ": " + this.error$1.toString();
        }
        
        return errorMessage;
    }
    /*

   //------------------------------------------------------------------------------

       private var test : Test;
       private var error : Error;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: TestFailure$,
        failedTest: failedTest,
        thrownException: thrownException,
        exceptionMessage: exceptionMessage,
        isFailure: isFailure,
        toString: toString,
        test$1: null,
        error$1: null,
        uses: [
            "flexunit.framework.AssertionFailedError"
        ]
    };
});

Ext.define("flexunit.framework.TestListener", function(TestListener) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /
package flexunit.framework
{

   /**
    * Listener Interface for test progress.
    * /
   public interface TestListener
   {
      function addError( test : Test, error : Error ) : void;
      function addFailure( test : Test, error : AssertionFailedError ) : void;
      function endTest( test : Test ) : void;
      function startTest( test : Test ) : void;
   }
}

============================================== Jangaroo part ==============================================*/
    return {};
});

Ext.define("flexunit.framework.TestResult", function(TestResult) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{

   import flexunit.utils.*;

   /**
    * A <code>TestResult</code> collects the results of an executing
    * Test. It is an instance of the Collecting Parameter pattern.
    * The test framework distinguishes between <i>failures</i> and <i>errors</i>.
    * A failure is anticipated and checked for with assertions. Errors are
    * unanticipated problems.
    *
    * @see flexunit.framework.Test
    * /
   public class TestResult
   {
      public var syncToFrame : Boolean = false;

      public*/
    function TestResult$() {
        this.failures$1 = AS3.cast(flexunit.utils.Collection, new flexunit.utils.ArrayList());
        this.errors$1 = AS3.cast(flexunit.utils.Collection, new flexunit.utils.ArrayList());
        this.listeners$1 = AS3.cast(flexunit.utils.Collection, new flexunit.utils.ArrayList());
        this.runTests$1 = 0;
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function run(testCase) /* : TestCase*/
    /* : void*/
    {
        /*if ( syncToFrame )
            FunctionCallQueue.getInstance().call( this, doRun, [ testCase ] );
         else
         */
        this.doRun$1(testCase);
    }
    /*

   //------------------------------------------------------------------------------

       //called by the AsyncTestHelper when it has either failed or received the callback
      public*/
    function continueRun(testCase) /* : TestCase*/
    /* : void*/
    {
        this.doContinue$1(testCase);
    }
    /*

   //------------------------------------------------------------------------------

      private*/
    function doRun(testCase) /* : TestCase*/
    /* : void*/
    {
        this.startTest(testCase);
        testCase.setTestResult(this);
        var protectedTestCase = /* : Protectable*/
            AS3.cast(flexunit.framework.Protectable, new flexunit.framework.ProtectedStartTestCase(testCase));
        var startOK = /* : Boolean*/
            this.doProtected$1(testCase, protectedTestCase);
        if (startOK) {
            this.doContinue$1(testCase);
        } else {
            this.endTest(testCase);
        }
        testCase.assertionsMade = this.assertionsMade;
    }
    /*

   //------------------------------------------------------------------------------
      private*/
    function doContinue(testCase) /* : TestCase*/
    /* : void*/
    {
        var protectedTestCase = /* : Protectable*/
            AS3.cast(flexunit.framework.Protectable, new flexunit.framework.ProtectedMiddleTestCase(testCase));
        this.doProtected$1(testCase, protectedTestCase);
        if (testCase.hasAsync()) {
            testCase.startAsync();
        } else {
            this.doFinish$1(testCase);
        }
    }
    /*

   //------------------------------------------------------------------------------

      private*/
    function doFinish(testCase) /* : TestCase*/
    /* : void*/
    {
        var protectedTestCase = /* : Protectable*/
            AS3.cast(flexunit.framework.Protectable, new flexunit.framework.ProtectedFinishTestCase(testCase));
        this.doProtected$1(testCase, protectedTestCase);
        this.endTest(testCase);
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function addError(test, /* : Test*/
    error) /* : Error*/
    /* : void*/
    {
        this.errors$1.addItem(new flexunit.framework.TestFailure(test, error));
        var iter = /* : Iterator*/
            this.listeners$1.iterator();
        while (iter.hasNext()) {
            var listener = /* : TestListener*/
                AS3.cast(flexunit.framework.TestListener, iter.next());
            listener.addError(test, error);
        }
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function addFailure(test, /* : Test*/
    error) /* : AssertionFailedError*/
    /* : void*/
    {
        this.failures$1.addItem(new flexunit.framework.TestFailure(test, error));
        var iter = /* : Iterator*/
            this.listeners$1.iterator();
        while (iter.hasNext()) {
            var listener = /* : TestListener*/
                AS3.cast(flexunit.framework.TestListener, iter.next());
            listener.addFailure(test, error);
        }
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function errorCount() /* : Number*/
    {
        return this.errors$1.length();
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function errorsIterator() /* : Iterator*/
    {
        return this.errors$1.iterator();
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function failureCount() /* : Number*/
    {
        return this.failures$1.length();
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function failuresIterator() /* : Iterator*/
    {
        return this.failures$1.iterator();
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function shouldStop() /* : Boolean*/
    {
        return this.stopTests$1;
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function stop(stopTests) /* : Boolean*/
    /* : void*/
    {
        this.stopTests$1 = stopTests;
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function wasSuccessful() /* : Boolean*/
    {
        return this.failureCount() == 0 && this.errorCount() == 0;
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function addListener(listener) /* : TestListener*/
    /* : void*/
    {
        if (this.listeners$1.contains(listener) == false)  {
            this.listeners$1.addItem(listener);
        }
        
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function removeListener(listener) /* : TestListener*/
    /* : void*/
    {
        if (this.listeners$1.contains(listener))  {
            this.listeners$1.removeItem(listener);
        }
        
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function runCount() /* : Number*/
    {
        return this.runTests$1;
    }
    /*

   //------------------------------------------------------------------------------

      private*/
    function doProtected(testCase, /* : Test*/
    protectable) /* : Protectable*/
    /* : Boolean*/
    {
        var success = /* : Boolean*/
            false;
        try {
            if (AS3.is(protectable, flexunit.framework.ProtectedMiddleTestCase)) {
                flexunit.framework.Assert.resetAssertionsMade();
            }
            protectable.protect();
            success = true;
        } catch (error) {
            if (AS3.is(error, AS3.Error)) {
                if (AS3.is(error, flexunit.framework.AssertionFailedError)) {
                    this.addFailure(testCase, AS3.cast(flexunit.framework.AssertionFailedError, error));
                } else {
                    this.addError(testCase, error);
                }
            } else  {
                throw error;
            }
            
        }
        if (AS3.is(protectable, flexunit.framework.ProtectedMiddleTestCase)) {
            this._localAssertionsMade$1 = flexunit.framework.Assert.assetionsMade;
        }
        return success;
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function startTest(test) /* : Test*/
    /* : void*/
    {
        var count = /* : Number*/
            test.countTestCases();
        this.runTests$1 = this.runTests$1 + count;
        var iter = /* : Iterator*/
            this.listeners$1.iterator();
        var listener;
        /* : TestListener*/
        while (iter.hasNext()) {
            listener = AS3.cast(flexunit.framework.TestListener, iter.next());
            listener.startTest(test);
        }
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function endTest(test) /* : Test*/
    /* : void*/
    {
        var iter = /* : Iterator*/
            this.listeners$1.iterator();
        var listener;
        /* : TestListener*/
        while (iter.hasNext()) {
            listener = AS3.cast(flexunit.framework.TestListener, iter.next());
            listener.endTest(test);
        }
    }
    /*

   //------------------------------------------------------------------------------
   
      public*/
    function get$assertionsMade() /* : Number*/
    {
        return this._localAssertionsMade$1;
    }
    /*
      
      private var stopTests : Boolean;
      private var failures : Collection;
      private var errors : Collection;
      private var listeners : Collection;
      private var runTests : Number;
      private var _localAssertionsMade : Number;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        syncToFrame: false,
        constructor: TestResult$,
        run: run,
        continueRun: continueRun,
        doRun$1: doRun,
        doContinue$1: doContinue,
        doFinish$1: doFinish,
        addError: addError,
        addFailure: addFailure,
        errorCount: errorCount,
        errorsIterator: errorsIterator,
        failureCount: failureCount,
        failuresIterator: failuresIterator,
        shouldStop: shouldStop,
        stop: stop,
        wasSuccessful: wasSuccessful,
        addListener: addListener,
        removeListener: removeListener,
        runCount: runCount,
        doProtected$1: doProtected,
        startTest: startTest,
        endTest: endTest,
        stopTests$1: false,
        failures$1: null,
        errors$1: null,
        listeners$1: null,
        runTests$1: NaN,
        _localAssertionsMade$1: NaN,
        __accessors__: {
            assertionsMade: {
                get: get$assertionsMade
            }
        },
        requires: [
            "AS3.Error"
        ],
        uses: [
            "flexunit.framework.Assert",
            "flexunit.framework.AssertionFailedError",
            "flexunit.framework.Protectable",
            "flexunit.framework.ProtectedFinishTestCase",
            "flexunit.framework.ProtectedMiddleTestCase",
            "flexunit.framework.ProtectedStartTestCase",
            "flexunit.framework.TestFailure",
            "flexunit.framework.TestListener",
            "flexunit.utils.ArrayList",
            "flexunit.utils.Collection"
        ]
    };
});

Ext.define("flexunit.framework.TestSuite", function(TestSuite) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{
   
   import flash.utils.*;
   import flash.utils.describeType;
   
   import flexunit.utils.*;
   
   /**
    * A <code>TestSuite</code> is a collection of tests.
    * @description
    * It runs a collection of test cases. Here is an example:
    * import samples.test.com.iterationtwo.*;
    * import flexunit.framework.*;
    * import flexunit.extensions.*;
    *
    * <code>
    * import samples.test.com.iterationtwo.*;
    * import flexunit.framework.*;
    * import flexunit.extensions.*;
    *
    * class samples.test.com.iterationtwo.AllTests
    * {
    *    public static function suite() : Test
    *    {
    *       var testSuite : TestSuite = new TestSuite();
    *
    *       testSuite.addTest( new TestSuite( TestMoney ) );
    *       testSuite.addTest( new TestSuite( TestMoneyWithSetUpTearDown ) );
    *       testSuite.addTest( TestSuiteWithSetUpTearDown.suite() );
    *
    *       return testSuite;
    *    }
    * }
    * </code>
    * A <code>TestRunner</code> can be used to run the tests in a <code>TestSuite</code><br><br>
    * The <code>TestSetup</code>class lets you add <code>setUp()</code> and <code>tearDown()</code>
    * methods to initialize and clean-up your test suites, as shown in this example:
    * <code>
    * import samples.test.com.iterationtwo.*;
    * import flexunit.framework.*;
    * import flexunit.extensions.*;
    *
    * class samples.test.com.iterationtwo.TestSuiteWithSetUpTearDown
    * {
    *    public static function suite() : Test
    *    {
    *       var testSuite : TestSuite = new TestSuite();
    *
    *       testSuite.addTest( new TestSuite( TestMoneyWithFailingTests ) );
    *
    *       var testSetUp = new TestSetup( testSuite );
    *
    *       testSetUp.setUp = setUp
    *       testSetUp.tearDown = tearDown;
    *
    *       return testSetUp;
    *    }
    *
    *    public static function setUp()
    *    {
    *    }
    *
    *    public static function tearDown()
    *    {
    *    }
    * }
    * </code>
    * You can create your own classes that extend <code>TestSetup</code> so that you can reuse common
    * test suite <code>setUp()</code> and <code>tearDown()</code> functionality across
    * test suites.
    *
    * @see flexunit.framework.Test
    * @see flexunit.flexui.TestRunner
    * @see flexunit.extensions.TestSetup
    * /
   public class TestSuite implements Test
   {

      /**
       * The TestSuite constructor, provided with the test to be run.
       * @param param The test class to be run. Can be a <code>TestCase</code>, another <code>TestSuite</code>
       * or any class that implements the <code>Test</code> interface.
       * /
       public*/
    function TestSuite$(param) /* : Object = null*/
    {
        if (arguments.length <= 0)  {
            param = null;
        }
        
        /*
           super()*/
        
        this.testArrayList = AS3.cast(flexunit.utils.Collection, new flexunit.utils.ArrayList());
        if (param == null)  {
            return;
        }
        
        if (AS3.is(param, Class)) {
            var c = /* : Class*/
                AS3.cast(Class, param);
            var newClass = /* : Test*/
                new c();
            this.addTestMethods$1(c, newClass);
            if (this.testArrayList.length() == 0) {
                var warningTestCase = /* : WarningTestCase*/
                    new flexunit.framework.WarningTestCase("No tests found in " + newClass.className);
                this.addTestToList$1(warningTestCase);
            }
        } else if (AS3.is(param, flexunit.framework.Test)) {
            this.addTestToList$1(AS3.cast(flexunit.framework.Test, param));
        } else {
            throw new AS3.Error("Can't handle constructor arg");
        }
    }
    /*

   //------------------------------------------------------------------------------
       public*/
    function toString() /* : String*/
    {
        return "TestSuite";
    }
    /*

   //------------------------------------------------------------------------------

   /**
    * Runs the test, populating <code>result</code> with the test results.
    * @param result The <code>TestResult</code> instance to be populated with the test results.
    * /
       public*/
    function runWithResult(result) /* : TestResult*/
    /* : void*/
    {
        this.runIter$1 = this.testArrayList.iterator();
        this.listener$1 = new flexunit.framework.TestSuiteTestListener(this, result);
        this.runNext(result);
    }
    /*

       public*/
    function runNext(result) /* : TestResult*/
    /* : void*/
    {
        if (this.runIter$1.hasNext()) {
            if (result.shouldStop()) {
                this.listener$1.pop();
                return;
            }
            //note that the TestSuiteListener will make sure that runNext is called only occasionally so that
            //recursion doesn't spiral out of control
            var test = /* : Test*/
                AS3.cast(flexunit.framework.Test, this.runIter$1.next());
            this.runTest$1(test, result);
        } else {
            this.listener$1.pop();
        }
    }
    /*

   //------------------------------------------------------------------------------

       private*/
    function runTest(test, /* : Test*/
    result) /* : TestResult*/
    /* : void*/
    {
        test.runWithResult(result);
    }
    /*

   //------------------------------------------------------------------------------

   /**
    * Adds a test to this test suite.
    * @param test The <code>Test</code> instance to be added.
    * /
       public*/
    function addTest(test) /* : Test*/
    /* : void*/
    {
        if (!(AS3.is(test, flexunit.framework.Test)))  {
            this.addTest(AS3.cast(flexunit.framework.Test, new flexunit.framework.WarningTestCase("Object instance passed to addTest does not implement Test interface")));
        }
        else  {
            this.addTestToList$1(test);
        }
        
    }
    /*

   //------------------------------------------------------------------------------

   /**
    * Wraps a test inside a <code>TestSuite</code> and adds it to this test suite.
    * @param testClass The <code>Class</code> that extends TestCase which will be added as a test suite.
    * /
       public*/
    function addTestSuite(testClass) /* : Class*/
    /* : void*/
    {
        //addClassTestsToList( testClass );
        this.addTestToList$1(new TestSuite(testClass));
    }
    /*

   //------------------------------------------------------------------------------

       private*/
    function addTestMethods(theClass, /* : Class*/
    newClass) /* : Test*/
    /* : void*/
    {
        var methodNames = /* : Array*/
            newClass.getTestMethodNames();
        for (var i = /* : uint*/
            0; i < methodNames.length; i++) {
            var method = /* : String*/
                String(methodNames[i]);
            this.addTestMethod$1(theClass, method);
        }
    }
    /*

   //------------------------------------------------------------------------------

       private*/
    function addTestMethod(theClass, /* : Class*/
    methodName) /* : String*/
    /* : void*/
    {
        this.addTestToList$1(this.createTestInstance$1(theClass, methodName));
    }
    /*

   //------------------------------------------------------------------------------

       private*/
    function createTestInstance(theClass, /* : Class*/
    methodName) /* : String*/
    /* : Test*/
    {
        var test = /* : Test*/
            new theClass();
        if (AS3.is(test, flexunit.framework.TestCase))  {
            AS3.cast(flexunit.framework.TestCase, test).methodName = methodName;
        }
        
        return test;
    }
    /*

   //------------------------------------------------------------------------------

   /**
    * Returns the names of the test methods for this test.
    * @return  Array of method names for this test
   * /
       public*/
    function getTestMethodNames() /* : Array*/
    {
        return (null);
    }
    /*

   //------------------------------------------------------------------------------

   /**
    * Returns all the tests in this test suite as an <code>Array</code>
    * @return An <code>Array</code> containing the tests in this test suite.
    * /
       public*/
    function getTests() /* : Array*/
    {
        return this.testArrayList.toArray();
    }
    /*

   //------------------------------------------------------------------------------

   /**
    * Iterates all tests in this suite to calculate the total number of tests in this suite.
    * @return A Number containing the count of all tests in this suite.
    * /
       public*/
    function countTestCases() /* : Number*/
    {
        var count = /* : Number*/
            0;
        var iter = /* : Iterator*/
            this.testArrayList.iterator();
        while (iter.hasNext()) {
            var test = /* : Test*/
                AS3.cast(flexunit.framework.Test, iter.next());
            count = count + test.countTestCases();
        }
        return count;
    }
    /*
         
   //------------------------------------------------------------------------------

   /**
    * The number of tests added to this suite.
    * @return A Number containing the number of tests added to this suite.
    * /
       public*/
    function testCount() /* : Number*/
    {
        return this.testArrayList.length();
    }
    /*

   //------------------------------------------------------------------------------

   /**
    * Returns the fully qualified class name.
    * @return The fully qualified class name.
    * /
       public*/
    function get$className() /* : String*/
    {
        return flash.utils.getQualifiedClassName(this).replace(/::/, ".");
    }
    /*


   //------------------------------------------------------------------------------

       private*/
    function addTestToList(test) /* : Test*/
    /* : void*/
    {
        this.testArrayList.addItem(test);
    }
    /*

   //------------------------------------------------------------------------------

       public var testArrayList : Collection;
       public var name : String;
       private var runIter : Iterator;
       private var listener : TestSuiteTestListener;

   }
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.framework.Test"
        ],
        constructor: TestSuite$,
        toString: toString,
        runWithResult: runWithResult,
        runNext: runNext,
        runTest$1: runTest,
        addTest: addTest,
        addTestSuite: addTestSuite,
        addTestMethods$1: addTestMethods,
        addTestMethod$1: addTestMethod,
        createTestInstance$1: createTestInstance,
        getTestMethodNames: getTestMethodNames,
        getTests: getTests,
        countTestCases: countTestCases,
        testCount: testCount,
        addTestToList$1: addTestToList,
        testArrayList: null,
        name: null,
        runIter$1: null,
        listener$1: null,
        __accessors__: {
            className: {
                get: get$className
            }
        },
        requires: [
            "AS3.Error",
            "flash.utils.getQualifiedClassName",
            "flexunit.framework.Reflective",
            "flexunit.framework.Test"
        ],
        uses: [
            "flexunit.framework.TestCase",
            "flexunit.framework.TestSuiteTestListener",
            "flexunit.framework.WarningTestCase",
            "flexunit.utils.ArrayList",
            "flexunit.utils.Collection"
        ]
    };
});

Ext.define("flexunit.framework.TestSuiteTestListener", function(TestSuiteTestListener) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{

   import flash.events.*;
   import flash.utils.*;
   public class TestSuiteTestListener implements TestListener
   {

       public*/
    function TestSuiteTestListener$(suite, /* : TestSuite*/
    result) /* : TestResult*/
    {
        this.suite$1 = suite;
        this.result$1 = result;
        result.addListener(AS3.cast(flexunit.framework.TestListener, this));
        //the Timer allows us to let frames progress and kill the deep recursion problem
        this.timer$1 = new flash.utils.Timer(5, 1);
        this.timer$1.addEventListener("timer", AS3.bind(this, "handleTimer"));
        if (listenerStack$static == null) {
            listenerStack$static = new Array();
        } else {
            result.removeListener(AS3.cast(flexunit.framework.TestListener, listenerStack$static[0]));
        }
        listenerStack$static.unshift(this);
    }
    /*

       public*/
    function endTest(test) /* : Test*/
    /* : void*/
    {
        if (listenerStack$static[0] != this) {
            //trace('TSTL.endTest ignored');
            return;
        }
        //BUG 114824 WORKAROUND
        this.timer$1.removeEventListener(flash.events.TimerEvent.TIMER, AS3.bind(this, "handleTimer"));
        this.timer$1 = new flash.utils.Timer(5, 1);
        this.timer$1.addEventListener(flash.events.TimerEvent.TIMER, AS3.bind(this, "handleTimer"));
        //END WORKAROUND
        this.timer$1.start();
    }
    /*

       public*/
    function handleTimer(event) /* : Event*/
    /* : void*/
    {
        this.suite$1.runNext(this.result$1);
    }
    /*

       public*/
    function pop() /* : void*/
    {
        var popped = /* : TestSuiteTestListener*/
            listenerStack$static.shift();
        //assertEquals(this, popped);
        this.result$1.removeListener(AS3.cast(flexunit.framework.TestListener, this));
        if (listenerStack$static.length > 0) {
            this.result$1.addListener(AS3.cast(flexunit.framework.TestListener, listenerStack$static[0]));
            listenerStack$static[0].endTest(null);
        }
    }
    /*

       public*/
    function addError(test, /* : Test*/
    error) /* : Error*/
    /* : void*/
    {}
    /*

       public*/
    function addFailure(test, /* : Test*/
    error) /* : AssertionFailedError*/
    /* : void*/
    {}
    /*

       public*/
    function startTest(test) /* : Test*/
    /* : void*/
    {}
    /*

       private static*/
    var listenerStack$static = /* : Array*/
        null;
    /*
       private var suite : TestSuite;
       private var result : TestResult;
       private var timer : Timer;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.framework.TestListener"
        ],
        constructor: TestSuiteTestListener$,
        endTest: endTest,
        handleTimer: handleTimer,
        pop: pop,
        addError: addError,
        addFailure: addFailure,
        startTest: startTest,
        suite$1: null,
        result$1: null,
        timer$1: null,
        requires: [
            "flash.events.Event",
            "flash.events.EventDispatcher",
            "flash.events.IEventDispatcher",
            "flash.events.TimerEvent",
            "flash.utils.Timer",
            "flash.utils.clearInterval",
            "flash.utils.setInterval",
            "flexunit.framework.TestListener"
        ]
    };
});

Ext.define("flexunit.framework.WarningTestCase", function(WarningTestCase) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.framework
{

   /**
    * A <code>WarningTestCase</code> is a framework generated
    * test cased used to highlight to the test writer that
    * there was a problem with their test.
    *
    * @see flexunit.framework.TestCase
    * /
   public class WarningTestCase extends TestCase
   {
      public*/
    function WarningTestCase$(message) /* : String*/
    {
        flexunit.framework.TestCase.prototype.constructor.call(this, "Warning: " + message);
        this.message$3 = message;
    }
    /*

   //------------------------------------------------------------------------------

      private*/
    function runTest() /* : void*/
    {
        flexunit.framework.Assert.fail(this.message$3);
    }
    /*

   //------------------------------------------------------------------------------

      override public*/
    function toString() /* : String*/
    {
        return this.methodName;
    }
    /*

   //------------------------------------------------------------------------------
      private var message : String;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "flexunit.framework.TestCase",
        constructor: WarningTestCase$,
        runTest$3: runTest,
        toString: toString,
        message$3: null,
        requires: [
            "flexunit.framework.Assert",
            "flexunit.framework.Reflective",
            "flexunit.framework.Test",
            "flexunit.framework.TestCase"
        ]
    };
});

Ext.define("flexunit.runner.BaseTestRunner", function(BaseTestRunner) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.runner
{

   import flexunit.framework.*;

   public class BaseTestRunner implements TestListener
   {

      public*/
    function startTest(test) /*:Test*/
    /*:void*/
    {
        this.testStarted(test);
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function endTest(test) /*:Test*/
    /*:void*/
    {
        this.testEnded(test);
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function addError(test, /*:Test*/
    error) /*:Error*/
    /*:void*/
    {
        this.testError(test, error);
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function addFailure(test, /*:Test*/
    error) /*:AssertionFailedError*/
    /*:void*/
    {
        this.testFailure(test, error);
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function testStarted(test) /* : Test*/
    /* : void*/
    {}
    /*

      public*/
    function testEnded(test) /* : Test*/
    /* : void*/
    {}
    /*

      public*/
    function testError(test, /* : Test*/
    error) /* : Error*/
    /* : void*/
    {}
    /*

      public*/
    function testFailure(test, /* : Test*/
    error) /* : AssertionFailedError*/
    /* : void*/
    {}
    /*
   }*/
    function BaseTestRunner$() {}
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.framework.TestListener"
        ],
        startTest: startTest,
        endTest: endTest,
        addError: addError,
        addFailure: addFailure,
        testStarted: testStarted,
        testEnded: testEnded,
        testError: testError,
        testFailure: testFailure,
        constructor: BaseTestRunner$,
        requires: [
            "flexunit.framework.TestListener"
        ]
    };
});

Ext.define("flexunit.textui.Printer", function(Printer) {
    /*package flexunit.textui {
import flexunit.framework.TestResult;

public interface Printer {
  function print( result:TestResult, runTime:Number ):void;
}
}

============================================== Jangaroo part ==============================================*/
    return {};
});

Ext.define("flexunit.textui.ResultPrinter", function(ResultPrinter) {
    /* /*
	Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.textui
{

   import flash.utils.*;
   import flexunit.framework.*;
   import flexunit.utils.*;

   public class ResultPrinter implements TestListener, Printer
   {
       public*/
    function ResultPrinter$() {
        /*
           super()*/
        
        this._progress$1 = "";
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function print(result, /*:TestResult*/
    runTime) /*:Number*/
    /*:void*/
    {
        if (this._progress$1.length > 0)  {
            AS3.trace(this._progress$1);
        }
        
        this.printHeader(runTime);
        this.printErrors(result);
        this.printFailures(result);
        this.printFooter$1(result);
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function printHeader(runTime) /*:Number*/
    /*:void*/
    {
        AS3.trace("Time: " + (runTime / 1000) + " seconds");
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function printErrors(result) /*:TestResult*/
    /*:void*/
    {
        this.printDefects(result.errorsIterator(), result.errorCount(), "error");
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function printFailures(result) /*:TestResult*/
    /*:void*/
    {
        this.printDefects(result.failuresIterator(), result.failureCount(), "failure");
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function printDefects(defects, /*:Iterator*/
    count, /*:Number*/
    type) /*:String*/
    /*:void*/
    {
        if (count == 0)  {
            return;
        }
        
        if (count == 1)  {
            AS3.trace("There was " + count + " " + type + ":");
        }
        else  {
            AS3.trace("There were " + count + " " + type + "s:");
        }
        
        for (var i = /*:Number*/
            1; defects.hasNext(); i++) {
            this.printDefect$1(AS3.cast(flexunit.framework.TestFailure, defects.next()), i);
        }
    }
    /*

   //------------------------------------------------------------------------------

       private*/
    function printDefect(defect, /*:TestFailure*/
    count) /*:Number*/
    /*:void*/
    {
        this.printDefectHeader$1(defect, count);
        this.printDefectTrace$1(defect);
    }
    /*

   //------------------------------------------------------------------------------

       private*/
    function printDefectHeader(defect, /*:TestFailure*/
    count) /*:Number*/
    /*:void*/
    {
        AS3.trace(count + ") " + defect.failedTest().toString());
    }
    /*

   //------------------------------------------------------------------------------

       private*/
    function printDefectTrace(defect) /*:TestFailure*/
    /*:void*/
    {
        AS3.trace("\t" + defect.exceptionMessage());
    }
    /*

   //------------------------------------------------------------------------------

       private*/
    function printFooter(result) /*:TestResult*/
    /*:void*/
    {
        if (result.wasSuccessful()) {
            AS3.trace("");
            AS3.trace("OK (" + result.runCount() + " test" + (result.runCount() == 1 ? "" : "s") + ")");
        } else {
            AS3.trace("");
            AS3.trace("FAILURES!!!");
            AS3.trace("Tests run: " + result.runCount() + ",  Failures: " + result.failureCount() + ",  Errors: " + result.errorCount());
        }
        AS3.trace("");
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function addError(test, /*:Test*/
    error) /*:Error*/
    /*:void*/
    {
        this.updateProgress$1("E");
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function addFailure(test, /*:Test*/
    error) /*:AssertionFailedError*/
    /*:void*/
    {
        this.updateProgress$1("F");
    }
    /*

   //------------------------------------------------------------------------------

       public*/
    function endTest(test) /*:Test*/
    /*:void*/
    {}
    /*

   //------------------------------------------------------------------------------

       public*/
    function startTest(test) /*:Test*/
    /*:void*/
    {
        this.updateProgress$1(".");
    }
    /*

   //------------------------------------------------------------------------------
       private*/
    function updateProgress(lr) /*:String*/
    /*:void*/
    {
        this._progress$1 = this._progress$1 + lr;
        if (this._progress$1.length > 40) {
            AS3.trace(this._progress$1);
            this._progress$1 = "";
        }
    }
    /*

   //------------------------------------------------------------------------------

       private var _progress:String;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.framework.TestListener",
            "flexunit.textui.Printer"
        ],
        constructor: ResultPrinter$,
        print: print,
        printHeader: printHeader,
        printErrors: printErrors,
        printFailures: printFailures,
        printDefects: printDefects,
        printDefect$1: printDefect,
        printDefectHeader$1: printDefectHeader,
        printDefectTrace$1: printDefectTrace,
        printFooter$1: printFooter,
        addError: addError,
        addFailure: addFailure,
        endTest: endTest,
        startTest: startTest,
        updateProgress$1: updateProgress,
        _progress$1: null,
        requires: [
            "AS3.trace",
            "flexunit.framework.TestListener",
            "flexunit.textui.Printer"
        ],
        uses: [
            "flexunit.framework.TestFailure"
        ]
    };
});

Ext.define("flexunit.textui.TestRunner", function(TestRunner) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.textui
{

   import flash.utils.*;
   import flash.utils.getTimer;
   import flexunit.framework.*;
   import flexunit.runner.BaseTestRunner;

   public class TestRunner extends BaseTestRunner
   {
       public*/
    function TestRunner$(onComplete, /*:Function=null*/
    printer) /*:Printer=null*/
    {
        flexunit.runner.BaseTestRunner.prototype.constructor.call(this);
        switch (arguments.length) {
            case 0:
                onComplete = null;
            case 1:
                printer = null;
        }
        if (printer == null)  {
            this.printer$2 = new flexunit.textui.ResultPrinter();
        }
        else  {
            this.printer$2 = printer;
        }
        
        this.testsComplete$2 = onComplete;
    }
    /*



   //------------------------------------------------------------------------------

       public static*/
    function run$static(test, /*:Test*/
    onComplete, /*:Function=null*/
    printer) /*:Printer=null*/
    /*:TestResult*/
    {
        switch (Math.max(arguments.length, 1)) {
            case 1:
                onComplete = null;
            case 2:
                printer = null;
        }
        return new TestRunner(onComplete, printer).doRun(test);
    }
    /*

   //------------------------------------------------------------------------------

       internal*/
    function doRun(test) /*:Test*/
    /*:TestResult*/
    {
        this.result$2 = new flexunit.framework.TestResult();
        this.result$2.addListener(AS3.cast(flexunit.framework.TestListener, this.printer$2));
        this.result$2.addListener(AS3.cast(flexunit.framework.TestListener, this));
        this.startTime$2 = flash.utils.getTimer();
        this.totalTestCount$2 = test.countTestCases();
        this.numTestsRun$2 = 0;
        test.runWithResult(this.result$2);
        return this.result$2;
    }
    /*

   //------------------------------------------------------------------------------

       override public*/
    function testEnded(test) /* : Test*/
    /*:void*/
    {
        if (++this.numTestsRun$2 == this.totalTestCount$2) {
            var endTime = /*:Number*/
                flash.utils.getTimer();
            var runTime = /*:Number*/
                endTime - this.startTime$2;
            this.printer$2.print(this.result$2, runTime);
            if (this.testsComplete$2 != null) {
                this.testsComplete$2();
            }
        }
    }
    /*

       private var printer : Printer;
       private var startTime : int;
       private var totalTestCount : int;
       private var numTestsRun : int;
       private var result:TestResult;
       private var testsComplete:Function;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "flexunit.runner.BaseTestRunner",
        constructor: TestRunner$,
        doRun: doRun,
        testEnded: testEnded,
        printer$2: null,
        startTime$2: 0,
        totalTestCount$2: 0,
        numTestsRun$2: 0,
        result$2: null,
        testsComplete$2: null,
        statics: {
            run: run$static
        },
        requires: [
            "flash.utils.getTimer",
            "flexunit.framework.TestListener",
            "flexunit.runner.BaseTestRunner"
        ],
        uses: [
            "flexunit.framework.TestResult",
            "flexunit.textui.ResultPrinter"
        ]
    };
});

Ext.define("flexunit.textui.XmlResultPrinter", function(XmlResultPrinter) {
    /*package flexunit.textui {
import flexunit.framework.AssertionFailedError;
import flexunit.framework.Test;
import flexunit.framework.TestCase;
import flexunit.framework.TestFailure;
import flexunit.framework.TestListener;
import flexunit.framework.TestResult;
import flexunit.framework.TestSuite;
import flexunit.textui.Printer;
import flexunit.utils.ArrayList;

public class XmlResultPrinter implements TestListener, Printer{

  private var resultXml:Array = null;
  private var mTests:Array =*/
    function mTests_() {
        this.mTests$1 = (new Array());
    }
    /*;
  private var mCurrentTest = null;
  private var mSuite:String = null;


  private var successfullTests:Array = null;

  public*/
    function XmlResultPrinter$(suitName) /*:String = null*/
    {
        mTests_.call(this);
        if (arguments.length <= 0)  {
            suitName = null;
        }
        
        this.resultXml$1 = new Array();
        this.successfullTests$1 = new Array();
        this.mTests$1 = new Array();
        this.mSuite$1 = suitName;
    }
    /*

  public*/
    function addError(test, /* : Test*/
    error) /* : Error*/
    /*:void*/
    {
        if (!this.mCurrentTest$1.mError) {
            this.mCurrentTest$1.mError = error;
        }
    }
    /*

  public*/
    function addFailure(test, /* : Test*/
    error) /* : AssertionFailedError*/
    /*:void*/
    {
        if (!this.mCurrentTest$1.mFailure) {
            this.mCurrentTest$1.mFailure = error;
        }
    }
    /*

  public*/
    function endTest(test) /* : Test*/
    /*:void*/
    {
        if (this.mCurrentTest$1 != null) {
            var endTime = new Date();
            this.mCurrentTest$1.mTime = endTime - this.mCurrentTest$1.mTime;
            this.mTests$1.push(this.mCurrentTest$1);
            this.mCurrentTest$1 = null;
        }
        this.successfullTests$1.push(test);
    }
    /*

  public*/
    function startTest(test) /* : Test*/
    /*:void*/
    {
        if (this.mSuite$1 == null)  {
            this.mSuite$1 = test.className;
        }
        
        this.mCurrentTest$1 = new Object();
        this.mCurrentTest$1.mName = test.className + " " + (AS3.as(test, flexunit.framework.TestCase)).methodName;
        this.mCurrentTest$1.mTime = new Date();
    }
    /*
  public*/
    function print(result, /*:TestResult*/
    runTime) /*:Number*/
    /*:void*/
    {
        this.resultXml$1.push('<?xml version="1.0" encoding="ISO-8859-1" ?>\n');
        this.resultXml$1.push('<testsuite errors="');
        this.resultXml$1.push(result.errorCount());
        this.resultXml$1.push('" failures="');
        this.resultXml$1.push(result.failureCount());
        this.resultXml$1.push('" name="');
        this.resultXml$1.push(this.mSuite$1);
        this.resultXml$1.push('" tests="');
        this.resultXml$1.push(result.runCount());
        this.resultXml$1.push('" time="');
        this.resultXml$1.push(runTime / 1000);
        this.resultXml$1.push('">\n');
        for (var i = 0; i < this.mTests$1.length; ++i) {
            var test = this.mTests$1[i];
            this.resultXml$1.push('    <testcase name="');
            this.resultXml$1.push(test.mName);
            this.resultXml$1.push('" time="');
            this.resultXml$1.push(test.mTime / 1000);
            this.resultXml$1.push('"');
            if (test.mError || test.mFailure) {
                this.resultXml$1.push('>\n');
                this.resultXml$1.push('        <');
                var defect;
                var tag;
                if (test.mError) {
                    defect = test.mError;
                    tag = "error";
                } else {
                    defect = test.mFailure;
                    tag = "failure";
                }
                this.resultXml$1.push(tag);
                this.resultXml$1.push(' message="');
                this.resultXml$1.push(this.escapeXml$1(defect.toString()));
                this.resultXml$1.push('" type=""');
                this.resultXml$1.push('/');
                this.resultXml$1.push('>\n');
                this.resultXml$1.push('    </testcase');
            } else {
                this.resultXml$1.push('/');
            }
            this.resultXml$1.push('>\n');
        }
        this.resultXml$1.push('</testsuite>');
    }
    /*

  private*/
    function escapeXml(str) /*:String*/
    /*:String*/
    {
        var escAmpRegEx = /&/g;
        var escLtRegEx = /</g;
        var escGtRegEx = />/g;
        var quotRegEx = /"/g;
        var aposRegEx = /'/g;
        var result = str.replace(escAmpRegEx, "&amp;").replace(escLtRegEx, "&lt;").replace(escGtRegEx, "&gt;").replace(quotRegEx, "&quot;").replace(aposRegEx, "&apos;");
        return result;
    }
    /*
  
  public*/
    function getXml() /*:String*/
    {
        return this.resultXml$1.join("");
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.framework.TestListener",
            "flexunit.textui.Printer"
        ],
        resultXml$1: null,
        mCurrentTest$1: null,
        mSuite$1: null,
        successfullTests$1: null,
        constructor: XmlResultPrinter$,
        addError: addError,
        addFailure: addFailure,
        endTest: endTest,
        startTest: startTest,
        print: print,
        escapeXml$1: escapeXml,
        getXml: getXml,
        requires: [
            "flexunit.framework.TestListener",
            "flexunit.textui.Printer"
        ],
        uses: [
            "flexunit.framework.TestCase"
        ]
    };
});

Ext.define("flexunit.utils.Collection", function(Collection) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.utils
{
   public interface Collection
   {
      function addItem( item:Object ) : Boolean;
      function clear() : void;
      function contains( item:Object ) : Boolean;
      function isEmpty() : Boolean;
      function iterator() : Iterator;
      function length() : Number;
      function getItemAt( index:Number ) : Object;
      function removeItem( item:Object ) : Boolean;
      function toArray() : Array;
   }
}

============================================== Jangaroo part ==============================================*/
    return {};
});

Ext.define("flexunit.utils.ArrayList", function(ArrayList) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.utils
{

   public class ArrayList implements Collection
   {

      public*/
    function ArrayList$() {
        this.items$1 = new Array();
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function addItem(item) /*:Object*/
    /*:Boolean*/
    {
        if (item == null)  {
            return false;
        }
        
        this.items$1.push(item);
        return true;
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function clear() /*:void*/
    {
        this.items$1 = new Array();
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function contains(item) /*:Object*/
    /*:Boolean*/
    {
        return (this.getItemIndex$1(item) > -1);
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function getItemAt(index) /*:Number*/
    /*:Object*/
    {
        return (this.items$1[index]);
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function iterator() /*:Iterator*/
    {
        return (AS3.cast(flexunit.utils.Iterator, new flexunit.utils.CollectionIterator(this)));
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function length() /*:Number*/
    {
        return this.items$1.length;
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function isEmpty() /*:Boolean*/
    {
        return (this.items$1.length == 0);
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function removeItem(item) /*:Object*/
    /*:Boolean*/
    {
        var itemIndex = /*:Number*/
            this.getItemIndex$1(item);
        if (itemIndex < 0)  {
            return false;
        }
        
        this.items$1.splice(itemIndex, 1);
        return true;
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function toArray() /* : Array*/
    {
        return this.items$1;
    }
    /*

   //------------------------------------------------------------------------------

      private*/
    function getItemIndex(item) /*:Object*/
    /*:Number*/
    {
        for (var i = /*:uint*/
            0; i < this.items$1.length; i++) {
            if (this.items$1[i] == item)  {
                return i;
            }
            
        }
        return -1;
    }
    /*

   //------------------------------------------------------------------------------

      private var items:Array;
   }
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.utils.Collection"
        ],
        constructor: ArrayList$,
        addItem: addItem,
        clear: clear,
        contains: contains,
        getItemAt: getItemAt,
        iterator: iterator,
        length: length,
        isEmpty: isEmpty,
        removeItem: removeItem,
        toArray: toArray,
        getItemIndex$1: getItemIndex,
        items$1: null,
        requires: [
            "flexunit.utils.Collection"
        ],
        uses: [
            "flexunit.utils.CollectionIterator",
            "flexunit.utils.Iterator"
        ]
    };
});

Ext.define("flexunit.utils.Iterator", function(Iterator) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.utils
{
   public interface Iterator
   {
      function hasNext():Boolean;
      function next():Object;
   }
}

============================================== Jangaroo part ==============================================*/
    return {};
});

Ext.define("flexunit.utils.CollectionIterator", function(CollectionIterator) {
    /* /*
   Copyright (c) 2008. Adobe Systems Incorporated.
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
     * Neither the name of Adobe Systems Incorporated nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
* /

package flexunit.utils
{

   public class CollectionIterator implements Iterator
   {
      public*/
    function CollectionIterator$(collection) /*:Collection*/
    {
        this.collection$1 = collection;
        this.cursor$1 = 0;
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function hasNext() /*:Boolean*/
    {
        return (this.cursor$1 < this.collection$1.length());
    }
    /*

   //------------------------------------------------------------------------------

      public*/
    function next() /*:Object*/
    {
        if (this.cursor$1 >= this.collection$1.length())  {
            throw new AS3.Error("Past end of collection");
        }
        
        return (this.collection$1.getItemAt(this.cursor$1++));
    }
    /*

   //------------------------------------------------------------------------------

      private var collection : Collection;
      private var cursor : Number;

   }
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flexunit.utils.Iterator"
        ],
        constructor: CollectionIterator$,
        hasNext: hasNext,
        next: next,
        collection$1: null,
        cursor$1: NaN,
        requires: [
            "AS3.Error",
            "flexunit.utils.Iterator"
        ]
    };
});

Ext.define("net.jangaroo.joounit.runner.BrowserRunner", function(BrowserRunner) {
    /*package net.jangaroo.joounit.runner {

import flash.utils.getQualifiedClassName;
import flash.utils.getTimer;

import flexunit.framework.AssertionFailedError;
import flexunit.framework.Test;
import flexunit.framework.TestListener;
import flexunit.framework.TestResult;
import flexunit.framework.TestSuite;
import flexunit.runner.BaseTestRunner;
import flexunit.textui.XmlResultPrinter;

import js.Element;

/**
 * TestRunner implementation for browsers (incl. phantomjs).
 * <p>
 * The following example uses phantomjs-joo-runner.js to invoke
 * <code>main</code> launching
 * <code>flexunit.framework.AllFrameworkTests</code>
 * with 30 seconds timeout:
 * <code>
 *   phantomjs joo/phantomjs-joo-runner.js '{
 *      testSuiteName:"flexunit.framework.AllFrameworkTests",
 *      jooMainClass:"net.jangaroo.joounit.runner.BrowserRunner",
 *      timeout:30000
 *   }'
 *   </code>
 * </p>
 * /
public class BrowserRunner extends BaseTestRunner {

  private var testResult:TestResult;
  private var printer:XmlResultPrinter;
  private var startTime : int;
  private var totalTestCount : int;
  private var numTestsRun : int;

  private var outputTestResult:Function =*/
    function outputTestResult_() {
        this.outputTestResult$2 = (function(result) /*:String*/
        /*:void*/
        {
            window["result"] = result;
            if (window["callPhantom"]) {
                window["callPhantom"](result);
            }
        });
    }
    /*;

  private var onComplete:Function =*/
    function onComplete_() {
        this.onComplete$2 = (function(testResult, /*:TestResult*/
        resultXml) /*:String*/
        /*:void*/
        {
            BrowserRunner.insertIntoDOM(resultXml);
        });
    }
    /*;

  private var testSuite:Function;

  /**
   * Config params are <code>testSuiteName:String</code>
   * <code>onComplete:function(TestResult,String)</code>
   * <code>outputTestResult:function(String)</code>
   * @param config
   * /
  public*/
    function BrowserRunner$(config) /*:Object*/
    {
        flexunit.runner.BaseTestRunner.prototype.constructor.call(this);
        outputTestResult_.call(this);
        onComplete_.call(this);
        this.testSuite$2 = config['testSuite'];
        if (config.onComplete) {
            this.onComplete$2 = config.onComplete;
        }
        if (config.outputTestResult) {
            this.outputTestResult$2 = config.outputTestResult;
        }
    }
    /*

  /**
   * Launch the tests.
   * @param testConfig
   * /
  public static*/
    function main$static(config) /*:Object*/
    /*:void*/
    {
        if (typeof config === "function") {
            config = {
                testSuite: config
            };
        }
        new BrowserRunner(config).run();
    }
    /*
  
  internal static*/
    function suiteNotFound$static(msg) /*:String*/
    {
        window.classLoadingError = msg;
        // this property is read by jangaroo-maven-plugin
        AS3.trace("[ERROR]", window.classLoadingError);
        BrowserRunner.exit(false);
    }
    /*

  internal*/
    function onSuiteLoaded() /* : void*/
    {
        try {
            if (typeof this.testSuite$2 == 'function' && typeof this.testSuite$2['suite'] == 'function') {
                AS3.trace("[INFO]", "running test suite " + flash.utils.getQualifiedClassName(this.testSuite$2));
                this.startTime$2 = flash.utils.getTimer();
                this.numTestsRun$2 = 0;
                /*
        const*/
                var suite = /*:TestSuite*/
                    this.testSuite$2['suite']();
                this.totalTestCount$2 = suite.countTestCases();
                suite.runWithResult(this.testResult$2);
            } else {
                BrowserRunner.suiteNotFound(flash.utils.getQualifiedClassName(this.testSuite$2) + " does not have a static method 'suite'.");
            }
        } catch (e) {
            if (AS3.is(e, AS3.Error)) {
                AS3.trace("[ERROR]", e);
                BrowserRunner.exit(false);
            } else  {
                throw e;
            }
            
        }
    }
    /*

  public*/
    function run() /*:void*/
    {
        this.printer$2 = new flexunit.textui.XmlResultPrinter(flash.utils.getQualifiedClassName(this.testSuite$2).replace(/::/, "."));
        this.testResult$2 = new flexunit.framework.TestResult();
        this.testResult$2.addListener(AS3.cast(flexunit.framework.TestListener, this.printer$2));
        this.testResult$2.addListener(AS3.cast(flexunit.framework.TestListener, this));
        this.onSuiteLoaded();
    }
    /*

  override public*/
    function testError(test, /*:Test*/
    error) /*:Error*/
    /*:void*/
    {
        AS3.trace("[ERROR]", "test in error", test, error, error["stack"]);
    }
    /*

  override public*/
    function testFailure(test, /*:Test*/
    error) /*:AssertionFailedError*/
    /*:void*/
    {
        AS3.trace("[ERROR]", "test failed", test, error, error["stack"]);
    }
    /*

  override public*/
    function testEnded(test) /*:Test*/
    /*:void*/
    {
        if (++this.numTestsRun$2 == this.totalTestCount$2) {
            var endTime = /*:Number*/
                flash.utils.getTimer();
            var runTime = /*:Number*/
                endTime - this.startTime$2;
            this.printer$2.print(this.testResult$2, runTime);
            AS3.trace('tests run: ' + this.testResult$2.runCount() + " (" + 'errors: ' + this.testResult$2.errorCount() + ", " + 'failures: ' + this.testResult$2.failureCount() + ")");
            AS3.trace(this.printer$2.getXml());
            /*

      const*/
            var resultXml = /*:String*/
                this.printer$2.getXml();
            this.outputTestResult$2(resultXml);
            this.onComplete$2(this.testResult$2, resultXml);
            BrowserRunner.exit(this.testResult$2.wasSuccessful());
        }
    }
    /*
  
  internal static*/
    function exit$static(b) /*:Boolean*/
    /* : void*/
    {
        /*
    const*/
        var exitFunc = /*:Function*/
            window['joo'] && window['joo']._exit;
        // see phantomjs-joo-config.js
        if (exitFunc) {
            exitFunc(b);
        }
    }
    /*

  public static*/
    function insertIntoDOM$static(resultXml) /*:String*/
    /*:void*/
    {
        /*
    const*/
        var pre = /*:Element*/
            window.document.createElement('pre');
        pre.appendChild(window.document.createTextNode(resultXml));
        window.document.body.appendChild(pre);
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "flexunit.runner.BaseTestRunner",
        testResult$2: null,
        printer$2: null,
        startTime$2: 0,
        totalTestCount$2: 0,
        numTestsRun$2: 0,
        testSuite$2: null,
        constructor: BrowserRunner$,
        onSuiteLoaded: onSuiteLoaded,
        run: run,
        testError: testError,
        testFailure: testFailure,
        testEnded: testEnded,
        statics: {
            main: main$static,
            suiteNotFound: suiteNotFound$static,
            exit: exit$static,
            insertIntoDOM: insertIntoDOM$static
        },
        requires: [
            "AS3.Error",
            "AS3.trace",
            "flash.utils.getQualifiedClassName",
            "flash.utils.getTimer",
            "flexunit.framework.TestListener",
            "flexunit.runner.BaseTestRunner"
        ],
        uses: [
            "flexunit.framework.TestResult",
            "flexunit.textui.XmlResultPrinter"
        ]
    };
});

Ext.define("net.jangaroo.joounit.runner.DefaultJooTestRunner", function(DefaultJooTestRunner) {
    /*package net.jangaroo.joounit.runner {

import flexunit.framework.AssertionFailedError;
import flexunit.framework.Test;
import flexunit.framework.TestListener;
import flexunit.framework.TestResult;
import flexunit.framework.TestSuite;
import flexunit.runner.BaseTestRunner;
import flexunit.textui.XmlResultPrinter;

import joo.getQualifiedObject;
import joo.require;

/**
 * A test runner that can be instantiated with several test class names and that will invoke a callback
 * when finished
 * /
public class DefaultJooTestRunner extends BaseTestRunner {

  private var testClassNames:Array;

  // names of the test classes the couldn't be found
  private var testClassLoadErrors:Vector.<String>;

  private var testName:String;
  private var testSuite:TestSuite;
  private var testResult:TestResult;
  private var printer:XmlResultPrinter;
  private var startTime : int;
  private var totalTestCount : int;
  private var numTestsRun : int;
  private var onComplete:Function;

  /**
   * Launch the tests.
   * @param config The test configuration like
   * <code>
   *  {
   *    "tests": ["com.mycompany.Test1", "com.mycompany.Test2"],
   *    "onComplete" : function(testSuite:TestSuite, testResult:TestResult, testResultXml:String) {...}
   *  }
   * </code>
   * /
  public static*/
    function main$static(config) /*:Object*/
    /*:void*/
    {
        var name = /*:String*/
            "DefaultJooTestRunner";
        var onComplete;
        /*:Function*/
        var testClassNames = /*:Array*/
            null;
        if (AS3.is(config, String)) {
            testClassNames = [
                AS3.as(config, String)
            ];
        } else if (AS3.is(config['tests'], Array)) {
            testClassNames = AS3.as(config['tests'], Array);
        }
        if (AS3.is(config['onComplete'], Function)) {
            onComplete = AS3.as(config['onComplete'], Function);
        }
        if (AS3.is(config['name'], String)) {
            name = config['name'];
        }
        if (testClassNames == null) {
            throw new AS3.Error("No test classes specified");
        }
        new DefaultJooTestRunner(name, testClassNames, onComplete).run();
    }
    /*



  public*/
    function DefaultJooTestRunner$(name, /*:String*/
    testClassNames, /*:Array*/
    onComplete) /*:Function = null*/
    {
        flexunit.runner.BaseTestRunner.prototype.constructor.call(this);
        if (arguments.length <= 2)  {
            onComplete = null;
        }
        
        this.testName$2 = name;
        this.testClassNames$2 = testClassNames;
        this.onComplete$2 = onComplete;
    }
    /*

  public*/
    function run() /*:void*/
    {
        AS3.trace("[INFO]", "Running tests: " + this.testClassNames$2);
        this.printer$2 = new flexunit.textui.XmlResultPrinter(this.testName$2);
        this.testResult$2 = new flexunit.framework.TestResult();
        this.testResult$2.addListener(AS3.cast(flexunit.framework.TestListener, this.printer$2));
        this.testResult$2.addListener(AS3.cast(flexunit.framework.TestListener, this));
        // load all test classes
        this.testClassLoadErrors$2 = new Vector$object();
        /*.<String>*/
        joo.require(this.testClassNames$2, AS3.bind(this, "onSuiteLoaded"));
    }
    /*

  // =================


  internal*/
    function onSuiteLoaded() /* : void*/
    {
        // check whether a class loading problem has occurred
        if (this.testClassLoadErrors$2.length > 0) {
            AS3.trace("[ERROR]", "Couldn't load test classes: " + this.testClassLoadErrors$2);
            DefaultJooTestRunner.exit(false);
            return;
        }
        // build a test suite from all test classes
        this.testSuite$2 = new flexunit.framework.TestSuite();
        for (var i = /*:int*/
            0; i < this.testClassNames$2.length; i++) {
            var qualifiedTestClassName = /*:String*/
                this.testClassNames$2[i];
            var testClass = /*:**/
                joo.getQualifiedObject(qualifiedTestClassName);
            if (typeof testClass['suite'] == 'function') {
                // there is a static function 'suite'
                // assuming that this returns a test suite instance
                this.testSuite$2.addTest(testClass['suite']());
            } else if (AS3.is(testClass, Class)) {
                // it's a test class
                this.testSuite$2.addTestSuite(testClass);
            } else //      var test:Test = new testClass();
            //      if( !(test is Test) ) {
            //        throw new Error("Class "+testClassName+" is not a test implementation");
            //      }
            //      testSuite.addTest(test);
            {
                throw new AS3.Error("Not a valid test class: " + qualifiedTestClassName);
            }
        }
        // run test suite
        try {
            this.startTime$2 = new Date().getTime();
            this.numTestsRun$2 = 0;
            this.totalTestCount$2 = this.testSuite$2.countTestCases();
            AS3.trace("[INFO]", "Running " + this.totalTestCount$2 + " tests");
            this.testSuite$2.runWithResult(this.testResult$2);
        } catch (e) {
            if (AS3.is(e, AS3.Error)) {
                AS3.trace("[ERROR]", e);
                DefaultJooTestRunner.exit(false);
            } else  {
                throw e;
            }
            
        }
    }
    /*

  override public*/
    function testError(test, /*:Test*/
    error) /*:Error*/
    /*:void*/
    {
        AS3.trace("[ERROR]", "test in error", test, error);
    }
    /*

  override public*/
    function testFailure(test, /*:Test*/
    error) /*:AssertionFailedError*/
    /*:void*/
    {
        AS3.trace("[ERROR]", "test failed", test, error);
    }
    /*

  override public*/
    function testEnded(test) /*:Test*/
    /*:void*/
    {
        if (++this.numTestsRun$2 == this.totalTestCount$2) {
            var endTime = /*:Number*/
                new Date().getTime();
            var runTime = /*:Number*/
                endTime - this.startTime$2;
            this.printer$2.print(this.testResult$2, runTime);
            /*


      const*/
            var testResultXml = /*:String*/
                this.printer$2.getXml();
            if (this.onComplete$2 != null) {
                this.onComplete$2(this.testSuite$2, this.testResult$2, testResultXml);
            } else {
                AS3.trace('Tests: ' + this.testResult$2.runCount() + " (" + 'Errors: ' + this.testResult$2.errorCount() + ", " + 'Failures: ' + this.testResult$2.failureCount() + ")");
                AS3.trace(this.printer$2.getXml());
            }
            DefaultJooTestRunner.exit(this.testResult$2.wasSuccessful());
        }
    }
    /*

  internal static*/
    function exit$static(success) /*:Boolean*/
    /*:void*/
    {
        if (success) {
            AS3.trace("[INFO]", "Finished test execution successfully");
        } else {
            AS3.trace("[ERROR]", "Finished test execution with error(s)");
        }
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "flexunit.runner.BaseTestRunner",
        testClassNames$2: null,
        testClassLoadErrors$2: null,
        testName$2: null,
        testSuite$2: null,
        testResult$2: null,
        printer$2: null,
        startTime$2: 0,
        totalTestCount$2: 0,
        numTestsRun$2: 0,
        onComplete$2: null,
        constructor: DefaultJooTestRunner$,
        run: run,
        onSuiteLoaded: onSuiteLoaded,
        testError: testError,
        testFailure: testFailure,
        testEnded: testEnded,
        statics: {
            main: main$static,
            exit: exit$static
        },
        requires: [
            "AS3.Error",
            "AS3.trace",
            "flexunit.framework.TestListener",
            "flexunit.runner.BaseTestRunner"
        ],
        uses: [
            "flexunit.framework.TestResult",
            "flexunit.framework.TestSuite",
            "flexunit.textui.XmlResultPrinter"
        ]
    };
});

